package com.webmethods.portal.portlet.wm_locationmap;

import com.webmethods.rtl.util.Debug;

import javax.swing.*;
import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.awt.image.PixelGrabber;
import java.awt.image.ColorModel;
import java.io.File;
import java.util.Random;

/**
 */
public class LocationMapUtils {

    public static final int MAX_CIRCLE_WIDTH = 15;
    private static Random m_random = new Random(System.currentTimeMillis());

    public static class Coords {
        public int latitude;
        public int longitude;
        public Color color;
        public double scale;    //between 0 & 1

        public Coords(int latitude, int longitude) {
            this.latitude = latitude;
            this.longitude = longitude;
        }
    }

    public static void generateMap(String origMap, String outMap, Coords [] coords) throws Exception {
        Image sourceImage = load(new File(origMap));
        sourceImage = toBufferedImage(sourceImage);
        Graphics2D graphics = (Graphics2D) sourceImage.getGraphics();
        int width = sourceImage.getWidth(null);
        int height = sourceImage.getWidth(null) / 2; //don't know why height is off by a factor of 4

        for (int i = 0; i < coords.length; i++) {
            Coords coord = coords[i];
            Point2D point = convertCoordsToPoint(coord.latitude, coord.longitude, width, height);
            drawPoint(graphics, (int) point.getX(), (int) point.getY(), coord.color, coord.scale);
        }

        graphics.dispose();
        save(sourceImage, new File(outMap));
    }

    /**
     * Convert lat / long to xy coords with the top left corner being (0,0)
     * where lat / long has the center of the map being (0,0)
     *
     * @param latitude  (between -90 & 90)
     * @param longitude (between -180 & 180)
     * @param width     of image
     * @param height    of image
     * @return a point to converted to the xy space of the graphics image
     */
    private static Point2D convertCoordsToPoint(int latitude, int longitude, int width, int height) {
        //figure out pct
        double convertedLongitude = convertCoord(360, longitude, width);
        double convertedLatitude = height - convertCoord(180, latitude, height);

        return new Point2D.Double(convertedLongitude, convertedLatitude);
    }

    private static double convertCoord(int range, int input, int scale) {
        double convert = input + (range / 2);
        convert /= range;
        return convert * scale;
    }

    private static void drawPoint(Graphics2D graphics2D, int x, int y, Color color, double scale) throws Exception {
        //int size = (int)(MAX_CIRCLE_WIDTH * scale);
        int size = MAX_CIRCLE_WIDTH;

        //need to randomize the x & y by a fraction of size
        int randomAmount = size * 1;
        int offsetX = getRandom(randomAmount) - randomAmount;
        int offsetY = getRandom(randomAmount) - randomAmount;
        x += offsetX;
        y += offsetY;

        Shape shape = new Ellipse2D.Double(x, y, size, size);
        Color saveColor = graphics2D.getColor();
        Composite composite = graphics2D.getComposite();
        graphics2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float)scale));
        graphics2D.setColor(color);
        graphics2D.fill(shape);
        graphics2D.setComposite(composite);

        //outline
        graphics2D.setColor(Color.BLACK);
        graphics2D.draw(shape);

        graphics2D.setColor(saveColor);
    }

    private static int getRandom(int range) {
        int a = m_random.nextInt();
        int i = Math.abs(a);
        i = i % range;
        return i;
    }

    private static Image load(File file) throws Exception {
        return Toolkit.getDefaultToolkit().createImage(file.getAbsolutePath());
    }

    private static void save(Image image, File file) throws Exception {
        ImageIO.write(toBufferedImage(image), "jpg", file);
    }

    private static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage) image;
        }

        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();

        int type = (hasAlpha(image)) ? BufferedImage.TYPE_INT_ARGB : BufferedImage.TYPE_INT_RGB;

        int height = image.getHeight(null);
        int width = image.getWidth(null);

        BufferedImage bimage = new BufferedImage(width, height, type);

        // Copy image to buffered image
        Graphics g = bimage.createGraphics();

        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();

        return bimage;
    }

    private static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage) image;
            return bimage.getColorModel().hasAlpha();
        }

        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient, unless it's a file based image producer...
        PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
            Debug.warn(e);
        }

        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return (cm == null) ? false : cm.hasAlpha();
    }
}
