/**
 ** $Workfile: Locationmap.java $
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_locationmap;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.bizPolicy.biz.IBizPolicyManager;
import com.webmethods.portal.bizPolicy.biz.IBizPolicyNames;
import com.webmethods.portal.bizPolicy.biz.dir.IPrincipalAttributeBizPolicy;
import com.webmethods.portal.event.IPortalEvent;
import com.webmethods.portal.framework.portlet.beans.JspPortletBean;
import com.webmethods.portal.mech.IMechanicsManager;
import com.webmethods.portal.portlet.wm_locationmap.mechanics.LocationmapMechanics;
import com.webmethods.portal.portlet.wm_locationmap.resources.LocationmapResources;
import com.webmethods.portal.service.dir.IDirPrincipal;
import com.webmethods.portal.service.meta2.IMetaContext;
import com.webmethods.portal.service.meta2.IMetaManager;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.util.Atomic;
import com.webmethods.portal.service.sql.DataManager;
import com.webmethods.portal.service.sql.ISqlWorker;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.rtl.util.*;

import java.awt.*;
import java.sql.ResultSet;
import java.util.*;
import java.util.List;

/**
 * Locationmap portlet implementation.
 *
 * @version $Revision: 1.0 $
 */
public class Locationmap extends JspPortletBean implements IPortalEvent.IListener {

    private static final LocationMapUtils.Coords INVALID_COORDS = new LocationMapUtils.Coords(Integer.MAX_VALUE, Integer.MAX_VALUE);

    public final static String PROPERTY_OUTPUTFILENAME = "outputFileName";
    public final static String PROPERTY_MAPWIDTH = "mapWidth";
    public final static String PROPERTY_MAPHEIGHT = "mapHeight";
    public final static String PROPERTY_COLORS = "colors";
    public final static String PROPERTY_HISTORYDAYS = "historyDays";
    public final static String PROPERTY_CITYNAME = "location";
    public final static String PROPERTY_ISDEBUGMODE = "isDebugMode";

    public void onEvent(IPortalEvent ev) {
        try {
            generateMap();
        } catch (Exception e) {
            Debug.warn(e);
        }
    }

    public Map getUserCoordsMap() throws PortalException {
        return getLocationmapMechanics().getUserCoordsMap();
    }

    public Map getCityStateCountryMap() throws PortalException {
        return getLocationmapMechanics().getCityStateCountryMap();
    }

    public synchronized void generateMap() throws Exception {

        //validation
        String location = getLocation();
        if (StringTools.isEmpty(location)) {
            throw new PortalException(LocationmapResources.class, "missing.prop.name");
        }

        //query database to get list of recent logins
        List recentLogins = getRecentLogins();
        List coordList = new ArrayList();

        String [] colors = StringTools.getArrayFromCsvString(getColors());
        long time = new Date().getTime();
        long range = getPropertyAsInt(PROPERTY_HISTORYDAYS, 1) * 86400 * 1000;

        for (Iterator it = recentLogins.iterator(); it.hasNext();) {
            Results result = (Results) it.next();
            LocationMapUtils.Coords coord = getCoordFromUserID(result.userID);
            if (coord == null || coord.latitude == Integer.MAX_VALUE) {
                continue;
            }

            //set color and scale
            double pct = (time - result.timestamp) / (double)range;
            coord.color = getColorForTimeStamp(colors, pct);
            coord.scale = 1 - pct;

            coordList.add(coord);
        }

        String resLocation = getResourceLocation("images/", false);
        String srcLocation = FileUtil.getRealLocation("webroot:" + PathUtil.appendPath(resLocation, "map.jpg"));
        String destLocation = FileUtil.getRealLocation("webroot:" + PathUtil.appendPath(resLocation, getOutputFileName()));

        LocationMapUtils.Coords[] coords = (LocationMapUtils.Coords[]) coordList.toArray(new LocationMapUtils.Coords[coordList.size()]);
        LocationMapUtils.generateMap(srcLocation, destLocation, coords);
    }

    private Color getColorForTimeStamp(String [] colors, double pct) {
        //now figure out which color from the array
        int index = (int)(pct * (double)(colors.length - 1));
        String color = colors[index].toLowerCase();
        Color result = (Color)ObjectUtil.getStaticProperty(Color.class, color);
        if (result == null) {
            Debug.info("Unable to find color: " + color);
            result = Color.BLACK;
        }

        return result;
    }

    private LocationmapMechanics getLocationmapMechanics() throws PortalException {
        IMechanicsManager mechanicsManager = (IMechanicsManager)PortalSystem.getMechanicsProvider();
        return (LocationmapMechanics)mechanicsManager.getMechanics(getPortletThingID());
    }

    private LocationMapUtils.Coords getCoordFromUserID(String userID) throws Exception {
        LocationMapUtils.Coords coords = (LocationMapUtils.Coords) getUserCoordsMap().get(userID);
        if (coords == null) {
            IMetaManager metaManager = (IMetaManager) PortalSystem.getMetaProvider();
            IMetaContext metaContext = metaManager.getDefaultContext();
            IThingID userThingID = Atomic.createValidThingId("user", Integer.parseInt(userID), metaContext);
            IDirPrincipal dirPrincipal = (IDirPrincipal) PortalSystem.getDirectoryProvider().lookup(userThingID);
            if (dirPrincipal == null) {
                Debug.debug("Unable to get dirPrincipal for thingID: " + userThingID.toString());
                return null;
            }

            IBizPolicyManager bizPolicyManager = (IBizPolicyManager) PortalSystem.getBizPolicyProvider();
            IPrincipalAttributeBizPolicy papBizPolicy = (IPrincipalAttributeBizPolicy) bizPolicyManager.getBizPolicy(IBizPolicyNames.PRINCIPAL_ATTRIBUTE);
            Map attributes = papBizPolicy.getAllAttributes(getPortletContext(), dirPrincipal.getDirectoryURI());
            String location = (String) attributes.get(getLocation());
            if (StringTools.isEmpty(location)) {
                Debug.debug("Unable to find location attribute for user: " + attributes);
                getUserCoordsMap().put(userID, INVALID_COORDS);
                return INVALID_COORDS;
            }

            coords = getCoordsFromLocation(location);
            if (coords == null) {
                return null;
            }
            getUserCoordsMap().put(userID, coords);
        }

        return coords;
    }

    private LocationMapUtils.Coords getCoordsFromLocation(String location) throws Exception {
        //check cache
        Integer[] results = (Integer[]) getCityStateCountryMap().get(location);
        if (results == null) {
            //strip out * and everything after a comma
            location = location.replace('*', ' ').trim();
            String city = findCityFromLocation(location);
            results = getLocationFromName(city);
            if (results == null) {
                String country = findCountryFromLocation(location);
                results = getLocationFromName(country);
            }
            if (results == null) {
                Debug.info("Unable to find coords for location: " + location);
                return null;
            }

            getCityStateCountryMap().put(location, results);
        }

        int latitude = results[0].intValue();
        int longitude = results[1].intValue();
        LocationMapUtils.Coords coords = new LocationMapUtils.Coords(latitude, longitude);
        return coords;
    }

    private String findCountryFromLocation(String location) {
        String country = null;
        int index = location.indexOf(',');
        if (index != -1) {
            country = location.substring(index);
        }

        index = location.indexOf('(');
        if (index != -1) {
            int endIndex = location.indexOf(')');
            country = location.substring(0, index) + location.substring(endIndex + 1);
            country = country.trim();
        }

        if (country == null) {
            country = location;
        }

        return country;
    }

    private String findCityFromLocation(String location) {
        String city = null;
        int index = location.indexOf(',');
        if (index != -1) {
            city = location.substring(0, index);
        }

        index = location.indexOf('(');
        if (index != -1) {
            city = location.substring(0, index);
        }

        if (city == null) {
            city = location;
        }

        return city;
    }

    private Integer[] getLocationFromName(String name) throws Exception {

        ISqlWorker worker = null;
        try {
            worker = DataManager.createWorker();
            ResultSet rs = worker.executeQuery("SELECT LATITUDE, LONGITUDE FROM PLACES " +
                    "WHERE UPPER(CITY) LIKE UPPER(?) OR UPPER(STATE) LIKE UPPER(?) OR UPPER(COUNTRY) LIKE UPPER(?)",
                    new Object[]{name, name, name}, null);

            Integer[] results = null;
            while (rs.next()) {
                double latitude = Double.parseDouble(rs.getString("LATITUDE"));
                double longitude = Double.parseDouble(rs.getString("LONGITUDE"));

                results = new Integer[]{new Integer((int) latitude), new Integer((int) longitude)};
                break;
            }

            return results;
        } finally {
            if (worker != null) {
                worker.release();
            }
        }

    }

    private List getRecentLogins() throws Exception {

        Integer numDays = new Integer(getPropertyAsInt(PROPERTY_HISTORYDAYS, 1));
        Long currentTime = new Long(new Date().getTime());

        ISqlWorker worker = null;
        try {
            long timestamp = currentTime.longValue() - (long)(86400 * 1000 * numDays.intValue());
            worker = DataManager.createWorker();
            ResultSet rs = worker.executeQuery("SELECT V.USERID, V.TIMESTAMP FROM ( " +
                    " SELECT ACTION, USERID, TIMESTAMP FROM TBLWEVENTS " +
                    " WHERE IDTYPE = (SELECT IDTYPE FROM TBLWEVENTTYPES WHERE EVENTTYPE = 'com.webmethods.portal.event.system.impl.LoginEvent') " +
                    " ) V  WHERE  V.TIMESTAMP > ?" +
                    " GROUP BY V.USERID, V.TIMESTAMP ", new Object[]{new Long(timestamp)}, null);

            List results = new ArrayList();
            while (rs.next()) {
                String userID = rs.getString("USERID");
                timestamp = rs.getLong("TIMESTAMP");

                results.add(new Results(userID, timestamp));
            }

            return results;
        } finally {
            if (worker != null) {
                worker.release();
            }
        }
    }

    private static class Results {
        public String userID;
        public long timestamp;

        public Results(String userID, long timestamp) {
            this.userID = userID;
            this.timestamp = timestamp;
        }
    }


    //--------------Properties-----------------


    /**
     * Sets the value of the 'outputFileName' property.
     *
     * @param value the value to use for the 'outputFileName' property
     */
    public void setOutputFileName(java.lang.String value) {
        setProperty(PROPERTY_OUTPUTFILENAME, value);
    }

    /**
     * Gets the value of the 'outputFileName' property.
     *
     * @return the value of the 'outputFileName' property
     */
    public java.lang.String getOutputFileName() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_OUTPUTFILENAME, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'mapWidth' property.
     *
     * @param value the value to use for the 'mapWidth' property
     */
    public void setMapWidth(java.lang.String value) {
        setProperty(PROPERTY_MAPWIDTH, value);
    }

    /**
     * Gets the value of the 'mapWidth' property.
     *
     * @return the value of the 'mapWidth' property
     */
    public java.lang.String getMapWidth() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_MAPWIDTH, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'mapHeight' property.
     *
     * @param value the value to use for the 'mapHeight' property
     */
    public void setMapHeight(java.lang.String value) {
        setProperty(PROPERTY_MAPHEIGHT, value);
    }

    /**
     * Gets the value of the 'mapHeight' property.
     *
     * @return the value of the 'mapHeight' property
     */
    public java.lang.String getMapHeight() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_MAPHEIGHT, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'colors' property.
     *
     * @param value the value to use for the 'colors' property
     */
    public void setColors(java.lang.String value) {
        setProperty(PROPERTY_COLORS, value);
    }

    /**
     * Gets the value of the 'colors' property.
     *
     * @return the value of the 'colors' property
     */
    public java.lang.String getColors() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_COLORS, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'historyDays' property.
     *
     * @param value the value to use for the 'historyDays' property
     */
    public void setHistoryDays(java.lang.String value) {
        setProperty(PROPERTY_HISTORYDAYS, value);
    }

    /**
     * Gets the value of the 'historyDays' property.
     *
     * @return the value of the 'historyDays' property
     */
    public java.lang.String getHistoryDays() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_HISTORYDAYS, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };

    /**
     * Sets the value of the 'cityName' property.
     *
     * @param value the value to use for the 'cityName' property
     */
    public void setLocation(java.lang.String value) {
        setProperty(PROPERTY_CITYNAME, value);
    }

    /**
     * Gets the value of the 'cityName' property.
     *
     * @return the value of the 'cityName' property
     */
    public java.lang.String getLocation() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_CITYNAME, "l");
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'isDebugMode' property.
     *
     * @param value the value to use for the 'isDebugMode' property
     */
    public void setIsDebugMode(java.lang.String value) {
        setProperty(PROPERTY_ISDEBUGMODE, value);
    }
    /**
     * Gets the value of the 'isDebugMode' property.
     *
     * @return the value of the 'isDebugMode' property
     */
    public java.lang.String getIsDebugMode() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_ISDEBUGMODE, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
}
