/**
 ** $Workfile: PageSpeed.java $
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_pagespeed;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.framework.IWebApplication;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.rtl.util.Debug;
import com.webmethods.rtl.util.StringTools;

/**
 * PageSpeed portlet implementation.
 *
 * @version $Revision: 1.0 $
 */
public class PageSpeed extends com.webmethods.portal.framework.portlet.beans.JspPortletBean {

    public final static String PROPERTY_LAYOUT = "Layout";
    public final static String PROPERTY_PREHTML = "prehtml";
    public final static String PROPERTY_POSTHTML = "posthtml";

    /**
     * Release any local variables so the object can be added back into the
     * object pool.
     */
    public void release() {
        super.release();
    }

    /**
     * Get the time from the preProccess to now.
     * Adjust it to be in seconds.
     *
     * @return the value of the difference between 'sUpdateSpeed' property
     *         and current time. In milliseconds.
     */
    public java.lang.String getUpdateSpeed() {
        try {
            // Get the preprocessor instance from the webApplication->IDispatchManager->preProcessor.
            PageSpeedProcessor psp = (PageSpeedProcessor) ((IWebApplication) PortalSystem.getWebAppProvider()).getDispatchManager().getComponent("pageSpeedProcessor");
            // Take the time difference from now.
            long lTime = System.currentTimeMillis() - psp.getTime(getRequest());
            // Convert it to sesconds out to the hundreds.
            long lMod = lTime % 10;
            // Round up if 5 or above.
            if (lMod > 4L)
                lTime = lMod + 10;
            // Move this to a double so you get a decimal value on the next operation.
            double dTime = lTime - lMod;
            // Note: Will not get a decimal if using longs in the operation.
            //       Dose not cast untill it gose across the equals.
            dTime /= 1000;
            // Make sure we have nnn.nn.
            String sMsg = Double.toString(dTime);
            if (sMsg.lastIndexOf('.') > sMsg.length() - 3)
                return sMsg + '0'; // no .nn (3 chars) then add one.
            return sMsg;
        } catch (Exception ex) {
            Debug.warn("PageSpeed.getUpdateSpeed(): " + ex.getLocalizedMessage());
        }
        return "0.00"; //Error if you get to this point.
    }

    /**
     * Sets the value of the 'Layout' property.
     *
     * @param value the value to use for the 'Layout' property
     */
    public void setLayout(java.lang.String value) {
        setProperty(PROPERTY_LAYOUT, value);
    }

    /**
     * Gets the value of the 'Layout' property.
     *
     * @return the value of the 'Layout' property
     */
    public java.lang.String getLayout() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_LAYOUT, "default");
        if (StringTools.isEmpty(strValue)) { //null check
            return "default";
        }
        return strValue;
    };
    /**
     * Sets the value of the 'Layout' property.
     *
     * @param value the value to use for the 'Layout' property
     */
    public void setPosthtml(java.lang.String value) {
        setProperty(PROPERTY_POSTHTML, value);
    }

    /**
     * Gets the value of the 'Layout' property.
     *
     * @return the value of the 'Layout' property
     */
    public java.lang.String getPosthtml() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_POSTHTML, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'Layout' property.
     *
     * @param value the value to use for the 'Layout' property
     */
    public void setPrehtlm(java.lang.String value) {
        setProperty(PROPERTY_PREHTML, value);
    }

    /**
     * Gets the value of the 'Layout' property.
     *
     * @return the value of the 'Layout' property
     */
    public java.lang.String getPrehtml() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_PREHTML, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };

}
