/**
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 **/
package com.webmethods.portal.portlet.wm_recentlogins.mechanics;

import com.webmethods.portal.event.system.ILoginEvent;
import com.webmethods.portal.mech.portlet.impl.PortletMechanics;
import com.webmethods.portal.service.dir.IDirPrincipal;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.type.IThingType;
import com.webmethods.portal.system.IComponentData;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.portal.system.init.InitializationException;
import com.webmethods.rtl.util.Debug;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * This is the implementation of your {@link com.webmethods.portal.mech.IMechanics}
 *
 * @version $Revision: 1.0 $
 */
public class RecentLoginsMechanics extends PortletMechanics implements ILoginEvent.IListener {

    private static final int MAX_SIZE = 10;

    private LinkedList m_recentLogins;

    public void init(IComponentData componentData, String phase) throws InitializationException {
        super.init(componentData, phase);
        m_recentLogins = new LinkedList();
        ILoginEvent.Listeners.add(this);
    }

    public void shutdown() {
        ILoginEvent.Listeners.remove(this);
        super.shutdown();
    }

    public void onEvent(ILoginEvent ev) {
        IThingID thingID = ev.getUserID();
        if (thingID != null && thingID.getTypeID() == IThingType.USER) {
            try {
                IDirPrincipal dirPrincipal = (IDirPrincipal) PortalSystem.getDirectoryProvider().lookup(thingID);
                if (dirPrincipal != null) {
                    m_recentLogins.remove(dirPrincipal);
                    m_recentLogins.addFirst(dirPrincipal);
                }
            } catch (Exception e) {
                Debug.warn(e);
            }
            if (m_recentLogins.size() > MAX_SIZE) {
                m_recentLogins.removeLast();
            }
        }
    }

    public List getRecentLogins() {
        return new ArrayList(m_recentLogins);
    }
}