/**
 ** $Workfile: Userhistory.java $
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_userhistory;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.framework.portlet.beans.JspPortletBean;
import com.webmethods.portal.service.dir.IDirPrincipal;
import com.webmethods.portal.service.meta2.IMetaContext;
import com.webmethods.portal.service.meta2.IMetaManager;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.thing.IThing;
import com.webmethods.portal.service.meta2.util.Atomic;
import com.webmethods.portal.service.sql.DataManager;
import com.webmethods.portal.service.sql.ISqlWorker;
import com.webmethods.portal.service.view.IListView;
import com.webmethods.portal.service.view.IView;
import com.webmethods.portal.service.view.impl.SimpleListView;
import com.webmethods.portal.system.IURI;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.rtl.util.StringTools;

import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
/**
 * Userhistory portlet implementation.
 *
 * @version    $Revision: 1.0 $
 */
public class Userhistory extends JspPortletBean {

    public final static String PROPERTY_CURRENTUSER = "currentUser";
    public final static String PROPERTY_NUMDAYS = "numDays";


    public IListView getUserHistory() throws Exception {
        Calendar interval = Calendar.getInstance();
        interval.set(interval.get(Calendar.YEAR), interval.get(Calendar.MONTH), interval.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
        interval.set(Calendar.MILLISECOND, 0);

        Long fromTime = new Long(interval.getTimeInMillis());

        String currentUser = getCurrentUser();
        if (StringTools.isEmpty(currentUser)) {
            return new SimpleListView();
        }

        IURI userURI = PortalSystem.getPortalSystem().acquireURI(currentUser);
        IDirPrincipal dirPrincipal = (IDirPrincipal)PortalSystem.getDirectoryProvider().lookup(userURI);
        IThingID dirThingID = dirPrincipal.getDirectoryPrincipalID();
        String userID = String.valueOf(dirThingID.getDbID().intValue());

        String numDays = '-' + getPropertyAsString(PROPERTY_NUMDAYS, "7");

        IMetaManager metaManager = (IMetaManager)PortalSystem.getMetaProvider();
        IMetaContext metaContext = metaManager.getDefaultContext();

        ISqlWorker worker = null;
        try {
            worker = DataManager.createWorker();
            ResultSet rs = worker.executeQuery("SELECT COUNT(V.THINGID_1) AS COUNT, " +
                    "V.THINGID_1, TBLTHING.IDTHINGXTYPE AS XTYPE " +
                        "FROM (" +
                            "SELECT USERID, THINGID_1, (FLOOR((TIMESTAMP - ?) / (86400 * 1000))) AS TIMESTAMP " +
                            " FROM TBLWEVENTS" +
                            " WHERE IDTYPE = (SELECT IDTYPE FROM TBLWEVENTTYPES WHERE EVENTTYPE = 'com.webmethods.portal.event.impl.GetEvent')" +
                        ") V" +
                    " INNER JOIN TBLTHING ON THINGID_1 = IDTHING" +
                    " WHERE USERID = ? AND TIMESTAMP > ?" +
                    " GROUP BY V.THINGID_1, TBLTHING.IDTHINGXTYPE ORDER BY COUNT DESC", new Object[]{fromTime, userID, numDays}, null);

            IListView listView = new SimpleListView();
            while (rs.next()) {

                String count = rs.getString("COUNT");
                int dbID = rs.getInt("THINGID_1");
                int xtypeID = rs.getInt("XTYPE");
                IThingID resultID = Atomic.createValidThingId(xtypeID, dbID, metaContext);
                IThing thing = resultID.getThing();

                Map props = new HashMap();
                props.put("count", count);
                props.put("name", thing.getName());
                props.put("description", thing.getDescription());
                props.put("uri", resultID);

                listView.add(new MapBasedView(props));
            }

            return listView;
        } finally {
            if (worker != null) {
                worker.release();
            }
        }
    }

    static class MapBasedView implements IView {
        Map m_properties;

        public MapBasedView(Map properties) {
            m_properties = properties;
        }

        public IURI getURI() throws PortalException {
            return null;
        }

        public Object getProperty(String name) throws PortalException {
            return m_properties.get(name);
        }

        public Map getProperties() throws PortalException {
            return m_properties;
        }

        public Collection getPropertyNames() throws PortalException {
            return m_properties.keySet();
        }
    }

    /**
     * Release any local variables so the object can be added back into the
     *  object pool.
     */
    public void release() {
        super.release();
    }

    /**
     * Sets the value of the 'currentUser' property.  
     *
     * @param value the value to use for the 'currentUser' property
     */
    public void setCurrentUser(java.lang.String value) {
        setProperty(PROPERTY_CURRENTUSER, value);
    }
    /**
     * Gets the value of the 'currentUser' property.  
     *
     * @return the value of the 'currentUser' property
     */
    public java.lang.String getCurrentUser() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_CURRENTUSER, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;    
        }
        return strValue;
    };
    /**
     * Sets the value of the 'numDays' property.  
     *
     * @param value the value to use for the 'numDays' property
     */
    public void setNumDays(java.lang.String value) {
        setProperty(PROPERTY_NUMDAYS, value);
    }
    /**
     * Gets the value of the 'numDays' property.  
     *
     * @return the value of the 'numDays' property
     */
    public java.lang.String getNumDays() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_NUMDAYS, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;    
        }
        return strValue;
    };
}
