/**
 ** Copyright 2005 webMethods, Inc. All Rights Reserved.
 **/
package app;

import com.webmethods.caf.faces.bean.BasePageBean;
import com.webmethods.caf.faces.bean.URLContentExportBean;
import com.webmethods.caf.faces.context.ContextUtils;
import com.webmethods.caf.faces.data.IContentProvider;
import com.webmethods.caf.faces.data.ITableContentProvider;
import com.webmethods.caf.faces.data.export.AtomEntriesExportProvider;
import com.webmethods.caf.faces.data.export.AtomFeedExportProvider;
import com.webmethods.caf.faces.data.export.CSVExportProvider;
import com.webmethods.caf.faces.data.object.ListTableContentProvider;
import com.webmethods.caf.faces.data.object.MapContentProvider;
import com.webmethods.caf.faces.data.object.NewsfeedContentProvider;
import com.webmethods.caf.faces.data.object.SelectableListTableContentProvider;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * BasicPageBean
 */
public class BasicPageBean extends BasePageBean {
    protected Object foo = "";
    protected List daList = new ArrayList();
    protected List meaList = Arrays.asList(new Object[] {
            new Object[] {"alice","apple","astros"},
            new Object[] {"bob","banana","brewers"},
            new Object[] {"carla","cantaloupe","cubs"},
            new Object[] {"dirk","dingleberries","dodgers"},
            new Object[] {"edna","eggplant","expos"},
            new Object[] {"frank","fruitcake","phillies"},
    });
    
    protected int first = 0;
    protected int rows = 3;
    
    protected ListTableContentProvider exportContent = new ListTableContentProvider(
        Arrays.asList(new Object[] {
                new Object[] {"one","apple","A is for Apple","2007/01/01 12:01 AM"},
                new Object[] {"two","boy","B is for Boy","2001/02/03 12:34 PM"},
                new Object[] {"three","cat","C is for Cat","2001/01/01 01:01 AM"},
                new Object[] {"four","dog","D is for Dog","2002/02/02 02:02 AM"},
                new Object[] {"five","egg","E is for Egg","2006/06/06 07:06 AM"},
                new Object[] {"six","fish","F is for Fish","1999/12/31 11:59 PM"},
                new Object[] {"seven","\u30df","\u30df is for \u30df\u30a4\u30db","2000/01/01 12:00 PM"},
        })
    );
    
    
    public BasicPageBean() {
        super();
    }
    
    public Object getFoo() {
        return foo;
    }
    
    public void setFoo(Object foo) {
        this.foo = foo;
    }
    
    public List getDaList() {
        return daList;
    }

    public void pushDaList() {
        daList.add(foo);
    }
    
    public void popDaList() {
        if (!daList.isEmpty())
            daList.remove(daList.size() - 1);
    }

    public int getFirst() {
        return first;
    }

    public void setFirst(int first) {
        this.first = first;
    }

    public int getRows() {
        return rows;
    }

    public void setRows(int rows) {
        this.rows = rows;
    }
    
    public List getMeaList() {
        return meaList;
    }
    
    public int getMeaFirst() {
        FacesContext context = FacesContext.getCurrentInstance();
        String s = (String) context.getExternalContext().getRequestParameterMap().get("meaFirst");
        if (s != null)
            return Integer.parseInt(s);
        return 0;
    }
    
    public int getMeaRows() {
        FacesContext context = FacesContext.getCurrentInstance();
        String s = (String) context.getExternalContext().getRequestParameterMap().get("meaRows");
        if (s != null)
            return Integer.parseInt(s);
        return 10;
    }
    
    public void exportFile() throws Exception {
        streamFileDataToResponse(new URLContentExportBean(getFacesContext().getExternalContext().getResource("/img/boston.jpg"), false));
    }
    
    public ListTableContentProvider getExportContent() {
        return exportContent;
    }
    
    public ITableContentProvider getCsvExportProvider() {
        Map<String,ValueBinding> keys = new LinkedHashMap<String,ValueBinding>();
        keys.put("Name", createValueBinding("#{row[1]}"));
        keys.put("Phrase", createValueBinding("#{row[2]}"));
        keys.put("Date", createValueBinding("#{row[3]}"));
        
        return new CSVExportProvider(getExportContent(), keys, "row");
    }
    
    public IContentProvider getAtomFeedProvider() {
        Map<String,String> feedProperties = new LinkedHashMap<String,String>();
        feedProperties.put("id", "export-sample");
        feedProperties.put("title", "Export Sample");
        feedProperties.put("author", "webMethods");
        
        MapContentProvider feed = new MapContentProvider(feedProperties);
        Map<String,ValueBinding> keys = AtomFeedExportProvider.createDefaultKeys(feed, "feed");
        
        return new AtomFeedExportProvider(feed, keys, "feed");
    }
    
    public ITableContentProvider getAtomEntriesProvider() {
        Map<String,ValueBinding> keys = new LinkedHashMap<String,ValueBinding>();
        keys.put("id", createValueBinding("#{row[1]}"));
        keys.put("title", createValueBinding("#{row[2]}"));
        keys.put("updated", createValueBinding("#{row[3]}"));
        
        return new AtomEntriesExportProvider(getExportContent(), keys, "row");
    }
    
    public NewsfeedContentProvider getTimBL() {
        return new NewsfeedContentProvider("http://dig.csail.mit.edu/breadcrumbs/blog/4/feed");
    }
    
    public NewsfeedContentProvider getTimBray() {
        return new NewsfeedContentProvider("http://www.tbray.org/ongoing/ongoing.atom");
    }
    
    public NewsfeedContentProvider getDaveWiner() {
        return new NewsfeedContentProvider("http://www.scripting.com/rss.xml");
    }
    
    protected String url;
    
    public String getUrl() {
        return this.url;
    }
    
    public void setUrl(String url) {
        this.url = url;
    }
    
    protected String gmapsApiKey;
    protected String address;
    protected double latitude;
    protected double longitude;
    protected int zoom;
    protected String mapType;
    
    public String getGmapsApiKey() {
        return gmapsApiKey;
    }
    
    public void setGmapsApiKey(String value) {
        gmapsApiKey = value;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public double getLatitude() {
        return latitude;
    }

    public void setLatitude(double latitude) {
        this.latitude = latitude;
    }
    
    public double getLatitudeReadOnly() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

    public void setLongitude(double longitude) {
        this.longitude = longitude;
    }

    public double getLongitudeReadOnly() {
        return longitude;
    }

    public int getZoom() {
        return zoom;
    }

    public void setZoom(int zoom) {
        this.zoom = zoom;
    }

    public int getZoomReadOnly() {
        return zoom;
    }

    public String getMapType() {
        return mapType;
    }

    public void setMapType(String mapType) {
        this.mapType = mapType;
    }

    public String getMapTypeReadOnly() {
        return mapType;
    }
    
    
    protected boolean markerOneSelected;
    protected double markerOneLatitude;
    protected double markerOneLongitude;

    public boolean isMarkerOneSelected() {
        return markerOneSelected;
    }

    public void setMarkerOneSelected(boolean markerOneSelected) {
        this.markerOneSelected = markerOneSelected;
    }

    public double getMarkerOneLatitude() {
        return markerOneLatitude;
    }

    public void setMarkerOneLatitude(double markerOneLatitude) {
        this.markerOneLatitude = markerOneLatitude;
    }

    public double getMarkerOneLongitude() {
        return markerOneLongitude;
    }

    public void setMarkerOneLongitude(double markerOneLongitude) {
        this.markerOneLongitude = markerOneLongitude;
    }
    
    
    protected SelectableListTableContentProvider dynamicMarkers;
    
    public SelectableListTableContentProvider getDynamicMarkers() {
        if (dynamicMarkers == null) {
            dynamicMarkers = new SelectableListTableContentProvider(new ArrayList(), "#{marker.address}", "marker");
            dynamicMarkers.setRowType(MyMarker.class);
        }
        return dynamicMarkers;
    }
    
    public List getDynamicMarkersReadOnly() {
        return getDynamicMarkers().getList();
    }
    
    public static class MyMarker implements Serializable {
        protected String address;
        protected double latitude;
        protected double longitude;
        
        public MyMarker() {
            //
        }
        
        public MyMarker(String address) {
            this();
            setAddress(address);
        }
        
        public MyMarker(String address, double latitude, double longitude) {
            this(address);
            setLatitude(latitude);
            setLongitude(longitude);
        }
        
        public String getAddress() {
            return address;
        }
        public void setAddress(String address) {
            this.address = address;
        }
        public double getLatitude() {
            return latitude;
        }
        public void setLatitude(double latitude) {
            this.latitude = latitude;
        }
        public double getLongitude() {
            return longitude;
        }
        public void setLongitude(double longitude) {
            this.longitude = longitude;
        }
        
        public boolean isSelected() {
            FacesContext context = ContextUtils.getFacesContext();
            SelectableListTableContentProvider provider = (SelectableListTableContentProvider)
                context.getApplication().createValueBinding("#{basicPage.dynamicMarkers}").getValue(context);
            return provider.getRowSelectedIds().contains(getAddress());
        }
    }
    
    
    protected SelectableListTableContentProvider threeDynamicMarkers;
    
    public SelectableListTableContentProvider getThreeDynamicMarkers() {
        if (threeDynamicMarkers == null) {
            threeDynamicMarkers = new SelectableListTableContentProvider(new ArrayList<MyMarker>(Arrays.asList(new MyMarker[] {
                    new MyMarker("Charleston, SC", 32.793053, -79.941294),
                    new MyMarker("St. Paul, MN", 44.944627, -93.102665),
                    new MyMarker("Las Vegas, NV", 36.233048, -115.246776),
            })), "#{marker.address}", "marker");
            threeDynamicMarkers.setRowType(MyMarker.class);
        }
        return threeDynamicMarkers;
    }
    

    protected SelectableListTableContentProvider multiMarkers;

    public SelectableListTableContentProvider getMultiMarkers() {
        if (multiMarkers == null) {
            multiMarkers = new SelectableListTableContentProvider(new ArrayList<MyMarker>(Arrays.asList(new MyMarker[] {
                    new MyMarker("Montgomery"),
                    new MyMarker("Juneau"),
                    new MyMarker("Phoenix"),
                    new MyMarker("Little Rock"),
                    new MyMarker("Sacramento"),
                    new MyMarker("Denver"),
                    new MyMarker("Hartford"),
                    new MyMarker("Dover"),
                    new MyMarker("Tallahassee"),
                    new MyMarker("notanaddress"),
                    new MyMarker("Atlanta"),
                    new MyMarker("Honolulu"),
                    new MyMarker("Boise"),
                    new MyMarker("Springfield"),
                    new MyMarker("Indianapolis"),
                    new MyMarker("Des Moines"),
                    new MyMarker("Topeka"),
                    new MyMarker("Frankfort"),
                    new MyMarker("Baton Rouge"),
                    new MyMarker("Augusta"),
                    new MyMarker("Annapolis"),
                    new MyMarker("Boston"),
                    new MyMarker("Lansing"),
                    new MyMarker("St Paul"),
                    new MyMarker("Jackson"),
                    new MyMarker("Jefferson City"),
                    new MyMarker("Helena"),
                    new MyMarker("Lincoln"),
                    new MyMarker("Carson City"),
                    new MyMarker("Concord"),
                    new MyMarker("Trenton"),
                    new MyMarker("Santa Fe"),
                    new MyMarker("Albany"),
                    new MyMarker("Raleigh"),
                    new MyMarker("Bismarck"),
                    new MyMarker("Columbus"),
                    new MyMarker("Oklahoma City"),
                    new MyMarker("Salem"),
                    new MyMarker("Harrisburg"),
                    new MyMarker("Providence"),
                    new MyMarker("Columbia"),
                    new MyMarker("Pierre"),
                    new MyMarker("Nashville"),
                    new MyMarker("Austin"),
                    new MyMarker("Salt Lake City"),
                    new MyMarker("Montpelier"),
                    new MyMarker("Richmond"),
                    new MyMarker("Olympia"),
                    new MyMarker("Charleston"),
                    new MyMarker("Madison"),
                    new MyMarker("Cheyenne"),
            })), "#{marker.address}", "marker");
            multiMarkers.setRowType(MyMarker.class);
        }
        return multiMarkers;
    }
}
