package app;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import com.ibm.icu.util.Calendar;
import com.ibm.icu.util.TimeZone;
import com.webmethods.caf.faces.data.calendar.CalendarEvent;
import com.webmethods.caf.faces.data.calendar.CalendarEventProvider;
import com.webmethods.caf.faces.data.calendar.ICalendarEvent;

public class TestCalendarEventProvider extends CalendarEventProvider {

	public Calendar m_calendar = null;

	public Calendar getCalendar() {
		if( m_calendar == null ) {
			m_calendar = Calendar.getInstance(TimeZone.getDefault(), Locale.getDefault() );
		}
		return m_calendar;
	}
	
	public List<ICalendarEvent> getEventList() {
		List<ICalendarEvent> list = new ArrayList<ICalendarEvent>();
		
		list.addAll(getPartialDayEvents());
		list.addAll(getAllDayEvents());
		Collections.sort(list);
		
		return list;
	}
	
	public List<ICalendarEvent> getPartialDayEvents() {
		
		Calendar cal = getCalendar();
		List<ICalendarEvent> list = new ArrayList<ICalendarEvent>();
		
		// get the the monday of this week
		cal.setTime( new Date() );
		cal.set(Calendar.DAY_OF_WEEK, 2);

		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 7 );
		Date startDate = cal.getTime();
		cal.add(Calendar.HOUR, 2 );
		Date endDate = cal.getTime();
		String header="Event Mon #1";
		String body = "7a-2h";
		CalendarEvent eventMon1 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventMon1);
		
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 10 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 1 );
		endDate = cal.getTime();
		header="Event Mon #2";
		body = "10a-1h";
		CalendarEvent eventMon2 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventMon2);
		
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 13 );
		startDate = cal.getTime();
		cal.add(Calendar.MINUTE, 30 );
		endDate = cal.getTime();
		header="Event Mon #3";
		body = "1p-1/2h";
		CalendarEvent eventMon3 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventMon3);

		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 13 );
		cal.add(Calendar.MINUTE, 30 );
		startDate = cal.getTime();
		cal.add(Calendar.MINUTE, 30 );
		endDate = cal.getTime();
		header="Event Mon #3.5";
		body = "1:30p-1/2h";
		CalendarEvent eventMon3Half = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventMon3Half);

		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 15 );
		cal.add(Calendar.MINUTE, 5 );
		startDate = cal.getTime();
		cal.add(Calendar.MINUTE, 30 );
		endDate = cal.getTime();
		header="Event Mon #4";
		body = "3p-1h-off boundary";
		CalendarEvent eventMon4 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventMon4);
		
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 15 );
		cal.add(Calendar.MINUTE, 5 );
		startDate = cal.getTime();
		cal.add(Calendar.MINUTE, 30 );
		endDate = cal.getTime();
		header="Event Mon #5";
		body = "3p-1h-off boundary";
		CalendarEvent eventMon5 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventMon5);

		// tues
		cal.setTime( new Date() );
		cal.set(Calendar.DAY_OF_WEEK, 3);

		// all day event (2a-10p)
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 2 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 20 );
		endDate = cal.getTime();
		header="Event Tues #1";
		body = "2a-20h";
		CalendarEvent eventTue1 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventTue1);

		// wed
		cal.setTime( new Date() );
		cal.set(Calendar.DAY_OF_WEEK, 4);

		// mid day til midnight (1p-11p)
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 13 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 10 );
		endDate = cal.getTime();
		header="Event Wed 1";
		body = "1p-12h";
		CalendarEvent eventWed1 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventWed1);

		// thurs
		cal.setTime( new Date() );
		cal.set(Calendar.DAY_OF_WEEK, 5);

		// 11am til 2am the next day (1p-2a)
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 11 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 14 );
		endDate = cal.getTime();
		header="Event Thur #1";
		body = "11a-2a";
		CalendarEvent eventThu1 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventThu1);

		// fri
		cal.setTime( new Date() );
		cal.set(Calendar.DAY_OF_WEEK, 6);

		// 2 same sized events in same slot
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 10 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 1 );
		endDate = cal.getTime();
		header="Event Fri #1";
		body = "10a-1h";
		CalendarEvent eventFri1 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventFri1);

		// second event - same size
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 10 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 1 );
		endDate = cal.getTime();
		header="Event Fri #2";
		body = "10a-1h";
		CalendarEvent eventFri2 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventFri2);
		
		// 2 different sized events in same slot
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 14 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 1 );
		endDate = cal.getTime();
		header="Event Fri #3";
		body = "2p-1h dup";
		CalendarEvent eventFri3 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventFri3);
		
		// 2 different sized events in same slot
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 14 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 2 );
		endDate = cal.getTime();
		header="Event Fri #4";
		body = "2p-1andhalf";
		CalendarEvent eventFri4 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventFri4);


		
		// an event that falls before a normal day display range
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 2 );
		startDate = cal.getTime();
		cal.add(Calendar.HOUR, 2 );
		endDate = cal.getTime();
		header="Event Fri #5";
		body = "2a-2hr";
		CalendarEvent eventFri5 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventFri5);

		// an event that falls after a normal day display range
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.add(Calendar.HOUR, 23 );
		startDate = cal.getTime();
		cal.add(Calendar.MINUTE, 30 );
		endDate = cal.getTime();
		header="Event Fri #6";
		body = "11p-30min";
		CalendarEvent eventFri6 = CalendarEvent.create(
				startDate,
				endDate,
				false,
				header,
				body,
				"");
		list.add(eventFri6);

		// sat
		cal.setTime( new Date() );
		cal.set(Calendar.DAY_OF_WEEK, 7);


		/*
		List<ICalendarEvent> list = new ArrayList<ICalendarEvent>();
		Date now = new Date();
		for( int ix=0; ix < 5; ix++ ) {
			Date startDate = (Date)now.clone();
			startDate.setDate(now.getDate()+ix);
			startDate.setHours(12+ix);
			Date endDate = (Date)startDate.clone();
			endDate.setHours( startDate.getHours()+1);

			String body = "Event #" + ix;
			String header="Header #" + ix;
			CalendarEvent event = CalendarEvent.create(
					startDate,
					endDate,
					false,
					header,
					body,
					"");

			Date startDate2 = (Date)now.clone();
			startDate2.setDate(now.getDate()+ix);
			startDate2.setHours(14+ix);
			Date endDate2 = (Date)startDate2.clone();
			endDate2.setHours( startDate2.getHours()+1);
			String body2 = "Event2 #" + ix;
			String header2="Header2 #" + ix;

			CalendarEvent event2 = CalendarEvent.create(
					startDate2,
					endDate2,
					false,
					header2,
					body2,
					"");
			list.add(event);
			list.add(event2);
		}
		*/
		
		return list;		
	}

	public List<ICalendarEvent> getAllDayEvents() {
		Calendar cal = getCalendar();
		List<ICalendarEvent> list = new ArrayList<ICalendarEvent>();

		
		// 2 Day event starting monday
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 2);
		
		Date startDate = cal.getTime();
		cal.add(Calendar.DATE, 1);
		Date endDate = cal.getTime();
		String header="2 Day Event";
		String body = "mon-2d";
		CalendarEvent event1 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event1);
	
		// 3 day event starting Tues
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 3);
		startDate = cal.getTime();
		cal.add(Calendar.DATE, 2);
		endDate = cal.getTime();
		header="3 Day Event";
		body = "tue-3d";
		CalendarEvent event2 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event2);
	
		// 2 day event starting Thurs
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 5);
		startDate = cal.getTime();
		cal.add(Calendar.DATE, 1);
		endDate = cal.getTime();
		header="2 Day Event";
		body = "thu-2d";
		CalendarEvent event3 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event3);

		// 1 day event starting Wed
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 4);
		startDate = cal.getTime();
		// start date is same date a end date
		endDate = cal.getTime();
		header="1 Day Event";
		body = "wed-1d";
		CalendarEvent event4 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event4);
		
		// 1 week event starting thursday last week
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 1);
		cal.add(Calendar.DAY_OF_WEEK, -3);
		startDate = cal.getTime();
		cal.add(Calendar.DATE, 6);
		endDate = cal.getTime();
		header="Week Long Event";
		body = "last thu-7d";
		CalendarEvent event5 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event5);

		// 4 day event starting Friday
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 6);
		startDate = cal.getTime();
		cal.add(Calendar.DATE, 3);
		endDate = cal.getTime();
		header="4 day Event";
		body = "fri-4d";
		CalendarEvent event6 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event6);

	
		// 2 week event starting Wed
		cal.setTime(new Date());
		cal.set(Calendar.MILLISECONDS_IN_DAY, 0);
		cal.set(Calendar.DAY_OF_WEEK, 4);
		startDate = cal.getTime();
		cal.add(Calendar.DATE, 13);
		endDate = cal.getTime();
		header="2 week Event";
		body = "wed-2week";
		CalendarEvent event7 = CalendarEvent.create(
				startDate,
				endDate,
				true,
				header,
				body,
				"");
		list.add(event7);

		return list;		

	}
}
