/**
 ** Copyright 2006 webMethods, Inc. All Rights Reserved.
 **/
package app.tree;

import com.webmethods.caf.faces.data.tree.INode;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.faces.FacesException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

/**
 * MSDNLib
 */
public class MSDNLib {
    protected static final String BASE = "http://msdn.microsoft.com/en-us/library/";
    protected static final String TOP = "ms310241(n)";
    
    protected static final Pattern RE_FIX_AMPS = Pattern.compile("& ");
    
    protected Map<String,List<MSDNLibNode>> nodes = new HashMap<String,List<MSDNLibNode>>(0x100);
    protected DocumentBuilderFactory documentBuilderFactory;
    
    public MSDNLib() {
        //
    }
    
    /**
     * Gets list of {@link INode} roots for MSDNLib tree.
     */
    public List<MSDNLibNode> getRoots() {
        List<MSDNLibNode> roots = new ArrayList<MSDNLibNode>(1);
        roots.add(new MSDNLibNode(this, null, "msdnlib", TOP, "MSDNLib", ""));
        return roots;
    }

    /**
     * Gets list of {@link INode} children for specified parent.
     */
    public List<MSDNLibNode> getChildren(MSDNLibNode parent) {
        List<MSDNLibNode> children = null;
        synchronized (nodes) {
            children = nodes.get(parent.getId());
        }
        if (children == null)
            children = createChildren(parent);
        
        List<MSDNLibNode> copy = new ArrayList<MSDNLibNode>(children.size());
        for (int i = 0; i < children.size(); i++) {
            MSDNLibNode src = children.get(i);
            copy.add(new MSDNLibNode(this, parent, src));
        }
        
        return copy;
    }
    
    /**
     * Gets list of {@link INode} roots for MSDNLib tree.
     */
    public List<MSDNLibNode> getCappedRoots(int cap) {
        List<MSDNLibNode> roots = new ArrayList<MSDNLibNode>(1);
        roots.add(new CappedMSDNLibNode(this, null, "msdnlib", TOP, "MSDNLib", "", cap));
        return roots;
    }

    /**
     * Gets list of {@link INode} children for specified parent.
     */
    public List<MSDNLibNode> getChildren(CappedMSDNLibNode parent) {
        if (parent.getCap() < 1)
            return Collections.EMPTY_LIST;
        
        List<MSDNLibNode> children = null;
        synchronized (nodes) {
            children = nodes.get(parent.getId());
        }
        if (children == null)
            children = createChildren(parent);
        
        List<MSDNLibNode> copy = new ArrayList<MSDNLibNode>(children.size());
        for (int i = 0; i < children.size(); i++) {
            MSDNLibNode src = children.get(i);
            copy.add(new CappedMSDNLibNode(this, parent, src, parent.getCap() - 1));
        }
        
        return copy;
    }
    
    //
    
    /**
     * Creates and caches list of {@link INode} children for specified parent.
     */
    protected List<MSDNLibNode> createChildren(MSDNLibNode parent) {
        List<MSDNLibNode> children = Collections.EMPTY_LIST;
        
        String src = parent.getSrc();
        if (src != null && src.length() > 0) {
            System.out.println("GET " + BASE + src);
            
            DocumentBuilder builder = getDocumentBuilder();
            try {
                Document doc = builder.parse(getInputSource(new URL(BASE + src)));
                
                //StringBuffer b = new StringBuffer(0x100);
                //debugElement(doc.getDocumentElement(), b, 0);
                //System.out.println(b);
                
                children = new ArrayList<MSDNLibNode>(0x10);
                for (Node n = doc.getDocumentElement().getFirstChild(); n != null; n = n.getNextSibling())
                    if (n.getNodeType() == Node.ELEMENT_NODE)
                        children.add(processElement((Element) n));
            
            } catch (IOException e) {
                throw new FacesException(e);
            } catch (SAXException e) {
                throw new FacesException(e);
            }
        }
        
        synchronized (nodes) {
            nodes.put(parent.getId(), children);
        }
        return children;
    }
    
    /**
     * Creates and caches {@link INode} for specified dom element.
     */
    protected MSDNLibNode processElement(Element e) {
        Element a = (Element) e.getFirstChild().getNextSibling();
        
        String name = e.getAttribute("name");
        String url = a.getAttribute("href");
        String title = a.getFirstChild().getNodeValue();
        
        // if <li> name attr is useless, use last bit of url for id
        String id = name;
        if (id == null || id.length() < 1 || "na".equals(id)) {
            id = url;
            if (id != null && id.startsWith(BASE))
                id = id.substring(BASE.length());
            if (id != null && id.endsWith(".aspx"))
                id = id.substring(0, id.length() - ".aspx".length());
        }
        
        // "na" == no children available
        String src = name;
        if ("na".equals(name))
            src = null;
        
        return new MSDNLibNode(this, null, id, src, title, url);
    }
    
    protected InputSource getInputSource(URL url) {
        try {
            InputStream in = url.openStream();
            InputStreamReader reader = new InputStreamReader(in, "UTF-8");
            
            // read input stream into buffer
            StringBuilder b = new StringBuilder(0x800);
            char[] ch = new char[0x400];
            int len = reader.read(ch);
            
            while (len != -1) {
                b.append(ch, 0, len);
                len = reader.read(ch);
            }
            
            reader.close();
            
            // fix up into xhtml
            String s = RE_FIX_AMPS.matcher(b).replaceAll("&amp;");
            
            // create input source
            InputSource src = new InputSource(new StringReader(s));
            src.setSystemId(url.toString());
            return src;
            
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new FacesException(e);
        }
    }
    
    /**
     * Creates new document builder.
     */
    protected DocumentBuilder getDocumentBuilder() {
        try {
            if (documentBuilderFactory == null) {
                documentBuilderFactory = DocumentBuilderFactory.newInstance();
                documentBuilderFactory.setValidating(false);
                documentBuilderFactory.setNamespaceAware(false);
            }
            return documentBuilderFactory.newDocumentBuilder();

        } catch (ParserConfigurationException e) {
            throw new FacesException(e);
        } catch (FactoryConfigurationError e) {
            throw new FacesException(e);
        }
    }

    /**
     * Debug helper.
     */
    protected void debugElement(Element e, StringBuffer b, int tabs) {
        b.append("\n");
        for (int i = 0; i < tabs; i++)
            b.append("  ");
        b.append("<").append(e.getTagName());
        
        NamedNodeMap attrs = e.getAttributes();
        for (int i = 0, l = attrs.getLength(); i < l; i++) {
            Attr attr = (Attr) attrs.item(i);
            b.append(" ").append(attr.getName()).append("=\"").append(attr.getValue()).append("\"");
        }
        
        b.append(">");
        
        NodeList children = e.getChildNodes();
        for (int i = 0, l = children.getLength(); i < l; i++) {
            Node child = children.item(i);
            if (child.getNodeType() == Node.ELEMENT_NODE)
                debugElement((Element) child, b, tabs + 1);
            else if (child.getNodeType() == Node.TEXT_NODE)
                b.append(child.getNodeValue());
        }
        
        b.append("\n");
        for (int i = 0; i < tabs; i++)
            b.append("  ");
        b.append("</").append(e.getTagName()).append(">");
    }
}
