package com.webmethods.service.web;

import java.util.HashMap;
import java.util.Map;

import com.webmethods.service.SampleItem;

public class SampleWebService {
	
	private static Map<Integer,SampleItem> items = new HashMap<Integer,SampleItem>();
	private static int nextId = 0;

	private SampleItem[] generateItems(int count, String itemName, String itemDescription, boolean generateChildren) {
		SampleItem[] output = new SampleItem[count];
		 
		// nothing
		for (int i = 0; i < count; i++) {
			output[i] = new SampleItem(
					nextId++, 
					itemName + " " + i, 
					itemDescription + " " + i);
			if (generateChildren) {
				output[i].setChildren(generateItems(i, "Child " + itemName, "Child " + itemDescription, false));
			}
		}		
		return output;
	}
	
	private void log(String s) {
		System.out.println(s);
	}
	
	public void initializeData(int count, String itemName, String itemDescription, boolean generateChildren) {
		log("initializeData(" + count + "," + itemName + "," + itemDescription + "," + generateChildren + ")");
		items.clear();
		SampleItem[] temp = generateItems(count, itemName, itemDescription, generateChildren);
		for (SampleItem item: temp) {
			items.put(item.getId(), item);
		}		
	}
	
	public SampleItem[] getAllItems() {
		log("getAllItems() returns: " + items.values());
		return items.values().toArray(new SampleItem[items.size()]);
	}

	public synchronized int getNextId() {
		int id = nextId++;
		log("getNextId() returns: " + id);
		return id;
	}
	
	public void addItem(SampleItem item) {
		log("addItem(" + item + ")");
		SampleItem oldItem = items.put(item.getId(), item);
		if (oldItem != null) {
			throw new IllegalArgumentException("Item with id " + item.getId() + " already exists");
		}
	}

	public SampleItem getItem(int id) {
		SampleItem sampleItem = items.get(id);
		log("getItem(" + id + ") returns " + sampleItem);
		if (sampleItem == null) {
			throw new IllegalArgumentException("Item with id " + id + " does not exist");
		}

		return sampleItem;
	}
	
	public void updateItem(SampleItem item) {
		log("updateItem(" + item + ")");
		SampleItem oldItem = items.put(item.getId(), item);
		if (oldItem == null) {
			throw new IllegalArgumentException("Item with id " + item.getId() + " does not exist");
		}
		// compare changes
	}
	
	public void deleteItem(int id) {
		log("deleteItem(" + id + ")");
		SampleItem item = items.remove(id);
		if (item == null) {
			throw new IllegalArgumentException("Item with id " + id + " does not exist");
		}
	}
	
	
}
