/**
 * 
 */
package com.webmethods.portlet.fabric_component;

/**
 * @author mitchw
 *
 */

import javax.portlet.PortletPreferences;

import com.webmethods.caf.portalclient.PortalClientException;
import com.webmethods.caf.portalclient.PortalClient;
import com.webmethods.caf.portalclient.alias.alias.IaliasPortType;
import com.webmethods.portal.PortalException;
import com.webmethods.portal.system.FabricSystem;

public class Fabric_componentBean extends com.webmethods.caf.faces.bean.BaseFacesPreferencesBean {

	/**
	 * List of portlet preference names
	 */
	public static final String[] PREFERENCES_NAMES = new String[] {
		"IntegrationServerInstalled", "WorkflowServerInstalled"
	};

	/**
	 * Create new preferences bean with list of preference names
	 */
	public Fabric_componentBean() {
		super(PREFERENCES_NAMES);
	}
	
	/**
	 * Call this method in order to persist
	 * Portlet preferences
	 */
	public void storePreferences() throws Exception {
		updatePreferences();
		PortletPreferences preferences = getPreferences();
		preferences.store();
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Boolean getWorkflowServerInstalled() throws Exception 
	{
		return new Boolean( FabricSystem.productExists( FabricSystem.WORKFLOW_ALIAS ));
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	/**
	 * Use the set preference method to update the 
	 * Workflow Server status in the fabric (insalled/uninstalled)
	 */
	public void setWorkflowServerInstalled(Boolean workflowServerInstalled) throws Exception 
	{
		setWorkflow( workflowServerInstalled );
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Boolean getIntegrationServerInstalled() throws Exception 
	{
		return new Boolean( FabricSystem.productExists( FabricSystem.MONITOR_ALIAS ));
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	/**
	 * Use the set preference method to update the 
	 * Integration Server status in the fabric (insalled/uninstalled)
	 */
	public void setIntegrationServerInstalled(Boolean integrationServerInstalled) throws Exception 
	{
		setIntegration( integrationServerInstalled );
	}
	
	protected void setWorkflow( Boolean bValue )
		throws PortalException, PortalClientException
	{
		// Set the fabric to look like it is installed.
		if( bValue.booleanValue() )
		{
			if( !isWorkflowInFabric() )
			    setAliasToThisPortlet( FabricSystem.WORKFLOW_ALIAS );
		}
		else
		{
			if( isWorkflowInFabric() )
				removeAliasFromThisPortlet( FabricSystem.WORKFLOW_ALIAS );
		}
	}

	protected void setIntegration( Boolean bValue )
		throws PortalException, PortalClientException
	{
		// Set the fabric to look like it is installed.
		if( bValue.booleanValue() )
		{
		    if( !isIntegrationInFabric())
			    setAliasToThisPortlet( FabricSystem.MONITOR_ALIAS );
		}
		else
		{
			if( isIntegrationInFabric())
				removeAliasFromThisPortlet( FabricSystem.MONITOR_ALIAS );
		}
	}
	
	protected void setAliasToThisPortlet( String sAlias )
		throws PortalException, PortalClientException
    {
		// Set the endpoint as inprocess to the local host
		PortalClient.initialize("inproc://localhost");
		IaliasPortType alias = PortalClient.getIaliasPortType( null );
		if( alias != null ) 
			alias.createalias( null, null, sAlias, false, "webm.caf.samples.fabric_component" );
    }

	protected void removeAliasFromThisPortlet( String sAlias )
		throws PortalException, PortalClientException
	{
		// Set the endpoint as inprocess to the local host
		PortalClient.initialize("inproc://localhost");
		IaliasPortType alias = PortalClient.getIaliasPortType( null );
		if( alias == null ) 
	        return;

		alias.removealias( sAlias, null, null, false, null );
	}

	protected boolean isWorkflowInFabric()
		throws PortalException
	{
		return FabricSystem.productExists( FabricSystem.WORKFLOW_ALIAS );
	}

	protected boolean isIntegrationInFabric()
		throws PortalException
	{
		return FabricSystem.productExists( FabricSystem.MONITOR_ALIAS );
	}
}