/**
 * 
 */
package com.webmethods.portlet.system_config;

import com.webmethods.portal.*;
import com.webmethods.portal.system.FabricSystem;
/**
 * @author mitchw
 *
 */

import javax.portlet.PortletPreferences;


public class System_configBean extends com.webmethods.caf.faces.bean.BaseFacesPreferencesBean {

	protected static final String NON_SSL_PROTOCOL = "http";
	protected static final String SSL_PROTOCOL = "https";
	/**
	 * List of portlet preference names
	 */
	public static final String[] PREFERENCES_NAMES = new String[] {
		"PortIS", "HostIS", "UseSSLIS", "ProtocolIS", "WorkflowDcsPort", "WorkflowDcsHost", "WorkflowDcsProtocol", "WorkflowDcsUseSSL", "WorkflowAuthPort", "WorkflowAuthHost", "WorflowAuthProtocol", "WorkflowAuthUseSSL", "WorkflowInBoxURL", "WorkflowServerPort", "WorkflowServerHost"
	};
	/**
	 * Create new preferences bean with list of preference names
	 */
	public System_configBean() {
		super(PREFERENCES_NAMES);
	}
	
	/**
	 * Call this method in order to persist
	 * Portlet preferences
	 */
	public void storePreferences() throws Exception {
		updatePreferences();
		PortletPreferences preferences = getPreferences();
		preferences.store();
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getHostIS() throws Exception {
		return (String) getPreferenceValue("HostIS", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setHostIS(String hostIS) throws Exception {
		setPreferenceValue("HostIS", hostIS);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Boolean getUseSSLIS() throws Exception {
		return (Boolean) getPreferenceValue("UseSSLIS", Boolean.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setUseSSLIS(Boolean useSSLIS) 
			throws Exception 
	{
		setPreferenceValue("UseSSLIS", useSSLIS);
		setProtocolIS( getProtocolText( useSSLIS));
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getProtocolIS() throws Exception {
		return (String) getPreferenceValue("ProtocolIS", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setProtocolIS(String protocolIS) throws Exception {
		setPreferenceValue("ProtocolIS", protocolIS);
	}

	protected String getProtocolText( Boolean bUseSSL )
	{
		if( bUseSSL.booleanValue())
			return SSL_PROTOCOL;
		return NON_SSL_PROTOCOL;
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Integer getPortIS() throws Exception {
		return (Integer) getPreferenceValue("PortIS", Integer.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setPortIS(Integer portIS) throws Exception {
		setPreferenceValue("PortIS", portIS);
	}

	/**
	 * Dev Notes: Accesser methods.  Need a string to bind to the JSF view
	 * for controlling the renderer.
	 */

    /**
     * Dev Notes: Use fabric common calls to enable and disable controls. 
     */
    public boolean getIntegrationUIComponentsNotInstalled() 
			throws PortalException 
	{
    	return !FabricSystem.productExists( FabricSystem.MONITOR_ALIAS );
    }

    public boolean getWorkflowUIComponentsNotInstalled() 
    		throws PortalException 
    {
        return !FabricSystem.productExists( FabricSystem.WORKFLOW_ALIAS );
    }

    public boolean getSaveEnabled()
    		throws PortalException 
    {
    	return !FabricSystem.productExists( FabricSystem.MONITOR_ALIAS )&&
        		!FabricSystem.productExists( FabricSystem.WORKFLOW_ALIAS );
    }
    public String getIntegrationServerDecriptionText() 
	throws PortalException 
	{
    	if( getWorkflowUIComponentsNotInstalled() )
    		return "Integration server component is not installed.";
    	return  "Please configure the fabric Integration server component.";
	}

    /**
     * Use this to tell the user that workflow server is not installed.
     * @return
     * @throws PortalException
     */
    public String getWorkflowServerDecriptionText() 
	throws PortalException 
	{
    	if( getWorkflowUIComponentsNotInstalled() )
    		return "Workflow server component is not installed.";
    	return  "Please configure the fabric workflow server component.";
	}
   

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Integer getWorkflowDcsPort() throws Exception {
		return (Integer) getPreferenceValue("WorkflowDcsPort", Integer.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowDcsPort(Integer workflowDcsPort) throws Exception {
		setPreferenceValue("WorkflowDcsPort", workflowDcsPort);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getWorkflowDcsHost() throws Exception {
		return (String) getPreferenceValue("WorkflowDcsHost", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowDcsHost(String workflowDcsHost) throws Exception {
		setPreferenceValue("WorkflowDcsHost", workflowDcsHost);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getWorkflowDcsProtocol() throws Exception {
		return (String) getPreferenceValue("WorkflowDcsProtocol", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowDcsProtocol(String workflowDcsProtocol) throws Exception {
		setPreferenceValue("WorkflowDcsProtocol", workflowDcsProtocol);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Boolean getWorkflowDcsUseSSL() throws Exception {
		return (Boolean) getPreferenceValue("WorkflowDcsUseSSL", Boolean.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowDcsUseSSL(Boolean workflowDcsUseSSL) throws Exception {
		setPreferenceValue("WorkflowDcsUseSSL", workflowDcsUseSSL);
		setWorkflowDcsProtocol( getProtocolText( workflowDcsUseSSL));
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getWorkflowAuthHost() throws Exception {
		return (String) getPreferenceValue("WorkflowAuthHost", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowAuthHost(String workflowAuthHost) throws Exception {
		setPreferenceValue("WorkflowAuthHost", workflowAuthHost);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Integer getWorkflowAuthPort() throws Exception {
		return (Integer) getPreferenceValue("WorkflowAuthPort", Integer.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowAuthPort(Integer workflowAuthPort) throws Exception {
		setPreferenceValue("WorkflowAuthPort", workflowAuthPort);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Boolean getWorkflowAuthUseSSL() throws Exception {
		return (Boolean) getPreferenceValue("WorkflowAuthUseSSL", Boolean.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowAuthUseSSL(Boolean workflowAuthUseSSL) throws Exception {
		setPreferenceValue("WorkflowAuthUseSSL", workflowAuthUseSSL);
		setWorkflowAuthProtocol( getProtocolText( workflowAuthUseSSL));
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getWorkflowInBoxURL() throws Exception {
		return (String) getPreferenceValue("WorkflowInBoxURL", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowInBoxURL(String workflowInBoxURL) throws Exception {
		setPreferenceValue("WorkflowInBoxURL", workflowInBoxURL);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public Integer getWorkflowServerPort() throws Exception {
		return (Integer) getPreferenceValue("WorkflowServerPort", Integer.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowServerPort(Integer workflowServerPort) throws Exception {
		setPreferenceValue("WorkflowServerPort", workflowServerPort);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getWorkflowServerHost() throws Exception {
		return (String) getPreferenceValue("WorkflowServerHost", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowServerHost(String workflowServerHost) throws Exception {
		setPreferenceValue("WorkflowServerHost", workflowServerHost);
	}

	/**
	 * The algorithm for this 'smart' preference getter is:
	 * 1) Check the Request Map (skip this step if it isn't a 'smart' preference)
	 * 2) Check the Member variable
	 * 3) Fall back to the PortletPreferences
	 */
	public String getWorkflowAuthProtocol() throws Exception {
		return (String) getPreferenceValue("WorkflowAuthProtocol", String.class);
	}

	/**
	 * Invoke {@link #storePreferences} to persist these changes
	 */
	public void setWorkflowAuthProtocol(String workflowAuthProtocol) throws Exception {
		setPreferenceValue("WorkflowAuthProtocol", workflowAuthProtocol);
	}
	
}