package com.webmethods.portal.portlet.wm_helloworld;

import javax.portlet.*;
import java.util.Locale;
import java.io.IOException;

public class HelloWorldPortlet extends GenericPortlet {

    private PortletContext pContext;

    public void init(PortletConfig config) throws PortletException {
        super.init(config);
        pContext = config.getPortletContext();

        pContext.log("HelloWorldPortlet: initilized!");
    }

    public void doView(RenderRequest request, RenderResponse response) throws PortletException {
        pContext.log("HelloWorldPortlet: doView! params: " + request.getParameterMap());

        String contentPage = getContentJSP(request);

        response.setContentType(request.getResponseContentType());
        if (contentPage != null && contentPage.length() != 0) {
            try {
                PortletRequestDispatcher dispatcher = pContext.getRequestDispatcher(contentPage);
                dispatcher.include(request, response);
            } catch (IOException e) {
                throw new PortletException("HelloWorldPortlet.doView exception", e);
            }
        }

    }

    public void doEdit(RenderRequest request, RenderResponse response) throws PortletException {
        pContext.log("HelloWorldPortlet: doEdit! params: " + request.getParameterMap());
        String editPage = getEditJSP(request);

        response.setContentType(request.getResponseContentType());
        if (editPage != null && editPage.length() != 0) {
            try {
                PortletRequestDispatcher dispatcher = pContext.getRequestDispatcher(editPage);
                dispatcher.include(request, response);
            } catch (IOException e) {
                throw new PortletException("HelloWorldPortlet.doEdit exception", e);
            }
        }
    }

    public void doHelp(RenderRequest request, RenderResponse response) throws PortletException {
        pContext.log("HelloWorldPortlet: doHelp! params: " + request.getParameterMap());
        String helpPage = getHelpJSP(request);

        response.setContentType(request.getResponseContentType());
        if (helpPage != null && helpPage.length() != 0) {
            try {
                PortletRequestDispatcher dispatcher = pContext.getRequestDispatcher(helpPage);
                dispatcher.include(request, response);
            } catch (IOException e) {
                throw new PortletException("HelloWord.doHelp exception", e);
            }
        }
    }

    public void processAction(ActionRequest request, ActionResponse actionResponse) throws PortletException, java.io.IOException {
        pContext.log("HelloWorldPortlet: processAction! params: " + request.getParameterMap());

        actionResponse.setRenderParameters(request.getParameterMap());
    }

    protected String getContentJSP(RenderRequest request) throws PortletException {
        PortletPreferences pref = request.getPreferences();
        String contentPage = pref.getValue("viewPage", "");
        return getLocalizedJSP(request.getLocale(), contentPage);
    }

    protected String getEditJSP(RenderRequest request) throws PortletException {
        PortletPreferences pref = request.getPreferences();
        String editPage = pref.getValue("editPage", "");
        return getLocalizedJSP(request.getLocale(), editPage);
    }

    protected String getHelpJSP(RenderRequest request) throws PortletException {
        PortletPreferences pref = request.getPreferences();
        String helpPage = pref.getValue("helpPage", "");
        return getLocalizedJSP(request.getLocale(), helpPage);
    }

    protected String getLocalizedJSP(Locale locale, String jspPath) {
        String realJspPath = jspPath;

        if (locale != null) {
            int separator = jspPath.lastIndexOf("/");
            String jspBaseDir = jspPath.substring(0, separator);
            String jspFileName = jspPath.substring(separator + 1);
            PortletContext pContext = getPortletContext();

            String searchPath = getJSPPath(jspBaseDir,
                    locale.toString(),
                    jspFileName);

            // search the requested JSP from the following location:
            // <ctxt_root>/<portlet_base_dir>_<language>_<country>/<jsp_file_name>
            if (pContext.getResourceAsStream(searchPath) != null) {
                realJspPath = searchPath;
            } else {
                // if the country code is not empty, try to search the
                // requested JSP from the following location:
                // <ctxt_root>/<portlet_base_dir>_<language>/<jsp_file_name>
                if (!locale.getCountry().equals("")) {
                    searchPath = getJSPPath(jspBaseDir,
                            locale.getLanguage(),
                            jspFileName);

                    if (pContext.getResourceAsStream(searchPath) != null) {
                        realJspPath = searchPath;
                    }
                }
            }
        }
        return realJspPath;
    }

    private String getJSPPath(String jspBaseDir,
                              String localeStr,
                              String jspFileName) {
        StringBuffer sb = new StringBuffer();
        sb.append(jspBaseDir)
                .append("_")
                .append(localeStr)
                .append("/")
                .append(jspFileName);
        return sb.toString();
    }
}
