/**
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_extended_my_favorites;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.bizPolicy.IContext;
import com.webmethods.portal.framework.portlet.beans.JspPortletBean;
import com.webmethods.portal.framework.propertyeditor.impl.ListTools;
import com.webmethods.portal.mech.IMechanicsManager;
import com.webmethods.portal.mech.storage.IBasicMechanics;
import com.webmethods.portal.mech.storage.IContainerMechanics;
import com.webmethods.portal.service.meta2.storage.ILink;
import com.webmethods.portal.service.meta2.thing.IThing;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.thing.IThingIDList;
import com.webmethods.portal.service.meta2.util.Atomic;
import com.webmethods.portal.service.sql.DataManager;
import com.webmethods.portal.service.sql.ISqlWorker;
import com.webmethods.portal.service.view.IView;
import com.webmethods.portal.system.IURI;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.rtl.util.Debug;
import com.webmethods.rtl.util.DynamicExpressionTools;
import com.webmethods.rtl.util.StringTools;

import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Extended_my_favorites portlet implementation.
 */
public class Extended_my_favorites extends JspPortletBean {
    public final static String PROPERTY_INCLUDE_FAVORITES_FOLDER = "includeFavoritesFolder";
    public final static String PROPERTY_IGNORED = "ignored";
    public final static String PROPERTY_SIZE = "size";

    static final Object SEPARATOR = new Object();

    static final String RECENT_PAGES_QUERY = "SELECT IDTHINGXTYPE, THINGID_1, SUM(CASE WHEN TIMESTAMP > ? THEN 20 WHEN TIMESTAMP > ? THEN 4 WHEN TIMESTAMP > ? THEN 2 ELSE 1 END) CNT FROM TBLWEVENTS E, TBLTHING t\n" +
            "\tWHERE E.IDTYPE = (SELECT IDTYPE FROM TBLWEVENTTYPES WHERE EVENTTYPE = 'com.webmethods.portal.event.impl.GetEvent') AND\n" +
            "               USERID = ? AND T.IDTHING = THINGID_1 \n" +
            "\tGROUP BY THINGID_1, IDTHINGXTYPE\n" +
            "\tORDER BY CNT DESC";

    static final String RECENT_PAGES_SESSION_KEY = "_recent_pages";

    // ui

    public List getList() throws PortalException {
        List recent = getRecentPages();

        if (getPropertyAsBoolean(PROPERTY_INCLUDE_FAVORITES_FOLDER)) {
            List ff = getFavoritesPages();
            if (!ff.isEmpty()) {
                List result = new ArrayList(recent.size() + ff.size());
                result.addAll(ff);
                result.add(SEPARATOR);
                result.addAll(recent);
                return result;
            }
        }

        return recent;
    }

    public String getFavortiesFolder() {
        return PortalSystem.USER_CURRENT_FAVORITES;
    }

    /**
     * List of IThingViews.
     */
    public List getFavoritesPages() throws PortalException {
        IThingID myFavoritesId = (IThingID) PortalSystem.getPortalSystem().acquireURI(PortalSystem.USER_CURRENT_FAVORITES);
        if (myFavoritesId != null) {
            IMechanicsManager mm = (IMechanicsManager) PortalSystem.getMechanicsProvider();
            IContainerMechanics cm = (IContainerMechanics) mm.getMechanics(myFavoritesId);
            IThingIDList childrenIds = cm.listChildren(myFavoritesId, getPortletContext().getUserID(), 0, -1);

            List children = new ArrayList(childrenIds.size());
            for (int i = 0; i < childrenIds.size(); i++) {
                IThingID id = childrenIds.getThingID(i);
                IBasicMechanics bm = (IBasicMechanics) mm.getMechanics(id);
                IView view = bm.view(id, getPortletContext().getUserID());
                children.add(view);
            }

            DynamicExpressionTools.sort(children, "{name}", getPortletContext());
            return children;
        }
        return Collections.EMPTY_LIST;
    }

    static class SessionCacheEntry {
        List recentPages = null;
        long timestamp = System.currentTimeMillis();

        public SessionCacheEntry(List recentPages) {
            this.recentPages = recentPages;
        }

        public boolean isExpired() {
            // every 10 min
            return timestamp < (System.currentTimeMillis() - 1000 * 60 * 10);
        }
    }

    /**
     * List of IThingViews.
     */
    public List getRecentPages() throws PortalException {
        try {
            // use simple session based cacheing not to hit db for each single page
            SessionCacheEntry cacheEntry = (SessionCacheEntry) getPortletContext().getAttribute(RECENT_PAGES_SESSION_KEY, IContext.SCOPE_SESSION);
            if (cacheEntry == null || cacheEntry.isExpired()) {
                cacheEntry = new SessionCacheEntry(loadRecentPages());
                getPortletContext().setAttribute(RECENT_PAGES_SESSION_KEY, cacheEntry, IContext.SCOPE_SESSION);
            }

            return cacheEntry.recentPages;
        }
        catch (Exception e) {
            Debug.warn(e);
            return Collections.EMPTY_LIST;
        }
    }

    // impl

    protected List loadRecentPages() throws Exception {
        // we would like to skip over the ignored list and the current list in the my favorites
        List ignorePages = ListTools.stringToList(getIgnored());
        for (int i = 0; i < ignorePages.size(); i++) {
            String ignorePage = (String) ignorePages.get(i);
            ignorePages.set(i, PortalSystem.getPortalSystem().acquireURI(ignorePage));
        }

        IThingID rootFolder = (IThingID) PortalSystem.getPortalSystem().acquireURI(getFavortiesFolder());
        IMechanicsManager mm = (IMechanicsManager) PortalSystem.getMechanicsProvider();
        IContainerMechanics cm = (IContainerMechanics) mm.getMechanics(rootFolder);
        IThingIDList thingIDList = cm.listChildren(rootFolder);
        for (int i = 0; i < thingIDList.size(); i++) {
            IThing thing = thingIDList.getThingID(i).getThing();
            if (thing instanceof ILink) {
                IURI shortcutTarget = PortalSystem.getPortalSystem().acquireURI(((ILink) thing).getURL());
                if (shortcutTarget instanceof IThingID) {
                    ignorePages.add(shortcutTarget);
                }
            }
            else {
                ignorePages.add(thingIDList.getThingID(i));
            }
        }

        List result = new ArrayList();

        ISqlWorker worker = null;
        try {
            worker = DataManager.createWorker();
            Integer userID = getPortletContext().getUserID().getDbID();
            long oneHour = System.currentTimeMillis() - 3600 * 1000;
            long oneDay = System.currentTimeMillis() - 86400 * 1000 * 1;
            long oneWeek = System.currentTimeMillis() - 86400 * 1000 * 7;
            ResultSet rs = worker.executeQuery(RECENT_PAGES_QUERY,
                    new Object[] {new Long(oneHour), new Long(oneDay), new Long(oneWeek), userID}, null);
            int count = 0;
            int size = Integer.parseInt(getSize());
            while (rs.next() && count < size) {
                IThingID itemID = Atomic.createValidThingId(rs, 1, getPortletContext().getUserID().getMetaContext());
                if (!ignorePages.contains(itemID)) {
                    itemID = (IThingID) PortalSystem.getPortalSystem().acquireURI(itemID.toString());
                    IBasicMechanics bm = (IBasicMechanics) mm.getMechanics(itemID);
                    IView view = bm.view(itemID, getPortletContext().getUserID());
                    result.add(view);
                    count++;
                }
            }
        }
        finally {
            if (worker != null) {
                worker.release();
            }
        }

        return result;
    }

    /**
     * Sets the value of the 'ignored' property.
     *
     * @param value the value to use for the 'ignored' property
     */
    public void setIgnored(java.lang.String value) {
        setProperty(PROPERTY_IGNORED, value);
    }
    /**
     * Gets the value of the 'ignored' property.
     *
     * @return the value of the 'ignored' property
     */
    public java.lang.String getIgnored() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_IGNORED, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
    /**
     * Sets the value of the 'size' property.
     *
     * @param value the value to use for the 'size' property
     */
    public void setSize(java.lang.String value) {
        setProperty(PROPERTY_SIZE, value);
    }
    /**
     * Gets the value of the 'size' property.
     *
     * @return the value of the 'size' property
     */
    public java.lang.String getSize() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_SIZE, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;
        }
        return strValue;
    };
}
