/**
 ** $Workfile: Related_pages.java $
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_related_pages;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.portlet.wm_related_pages.mechanics.Related_pagesMechanics;
import com.webmethods.portal.mech.IMechanicsManager;
import com.webmethods.portal.mech.view.IViewableMechanics;
import com.webmethods.portal.mech.storage.IContainerMechanics;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.thing.IThingIDList;
import com.webmethods.portal.service.meta2.thing.IThing;
import com.webmethods.portal.service.meta2.storage.ILink;
import com.webmethods.portal.service.meta2.util.Atomic;
import com.webmethods.portal.service.sql.ISqlWorker;
import com.webmethods.portal.service.sql.DataManager;
import com.webmethods.portal.service.cache.ICache;
import com.webmethods.portal.service.cache.ICacheKey;
import com.webmethods.portal.service.cache.ICacheEntry;
import com.webmethods.portal.service.view.IView;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.portal.system.IURI;
import com.webmethods.portal.framework.propertyeditor.impl.ListTools;
import com.webmethods.rtl.util.StringTools;
import com.webmethods.rtl.util.Debug;
import com.webmethods.rtl.util.DynamicExpressionTools;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.sql.ResultSet;
/**
 * Related_pages portlet implementation.
 *
 * @version    $Revision: 1.0 $
 */
public class Related_pages extends com.webmethods.portal.framework.portlet.beans.JspPortletBean {

    public final static String PROPERTY_IGNORED = "ignored";

    public final static String PROPERTY_STYLE = "style";
    public final static String PROPERTY_SIZE = "size";

    public List getList() throws PortalException {
        // validate current resource
        IThingID currentResourceID = (IThingID) PortalSystem.getPortalSystem().acquireURI("current.resource");
        if (currentResourceID == null) {
            return Collections.EMPTY_LIST;
        }

        // check cache first
        IMechanicsManager mm = (IMechanicsManager) PortalSystem.getMechanicsProvider();
        Related_pagesMechanics rpm = (Related_pagesMechanics) mm.getMechanics(getPortletThingID());
        List relatedPages = rpm.getRelatedPages(currentResourceID);
        if (relatedPages == null) {
            rpm.loadRelatedPages(currentResourceID, getIgnored());
            return Collections.EMPTY_LIST;
        }

        // get right list size
        if (relatedPages.size() > 0) {
            int size = Integer.parseInt(getSize());
            size = Math.min(size, relatedPages.size());
            relatedPages = relatedPages.subList(0, size);
        }

        // get list of IView instances
        List result = new ArrayList();
        IThingID userID = getPortletContext().getUserID();
        for (int i = 0; i < relatedPages.size(); i++) {
            IURI itemURI = (IURI) PortalSystem.getPortalSystem().acquireURI((String) relatedPages.get(i));
            if (itemURI instanceof IThingID) {
                IViewableMechanics viewMech = (IViewableMechanics) mm.getMechanics(itemURI);
                IView itemView = viewMech.view(itemURI, userID);
                result.add(itemView);
            }
        }

        // finally sort by name
        DynamicExpressionTools.sort(result, "{name}", getPortletContext());
        return result;
    }

    /**
     * Sets the value of the 'ignored' property.  
     *
     * @param value the value to use for the 'ignored' property
     */
    public void setIgnored(java.lang.String value) {
        setProperty(PROPERTY_IGNORED, value);
    }
    /**
     * Gets the value of the 'ignored' property.  
     *
     * @return the value of the 'ignored' property
     */
    public java.lang.String getIgnored() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_IGNORED, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;    
        }
        return strValue;
    };
    /**
     * Sets the value of the 'style' property.  
     *
     * @param value the value to use for the 'style' property
     */
    public void setStyle(java.lang.String value) {
        setProperty(PROPERTY_STYLE, value);
    }
    /**
     * Gets the value of the 'style' property.  
     *
     * @return the value of the 'style' property
     */
    public java.lang.String getStyle() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_STYLE, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;    
        }
        return strValue;
    };
    /**
     * Sets the value of the 'size' property.  
     *
     * @param value the value to use for the 'size' property
     */
    public void setSize(java.lang.String value) {
        setProperty(PROPERTY_SIZE, value);
    }
    /**
     * Gets the value of the 'size' property.  
     *
     * @return the value of the 'size' property
     */
    public java.lang.String getSize() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_SIZE, null);
        if (StringTools.isEmpty(strValue)) { //null check
            return null;    
        }
        return strValue;
    };
}
