/**
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 **/
package com.webmethods.portal.portlet.wm_related_pages.mechanics;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.framework.propertyeditor.impl.ListTools;
import com.webmethods.portal.service.cache.ICache;
import com.webmethods.portal.service.cache.ICacheEntry;
import com.webmethods.portal.service.cache.ICacheKey;
import com.webmethods.portal.service.cache.ICacheManager;
import com.webmethods.portal.service.cache.impl.DependentCache;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.util.Atomic;
import com.webmethods.portal.service.sql.DataManager;
import com.webmethods.portal.service.sql.ISqlWorker;
import com.webmethods.portal.system.IComponentData;
import com.webmethods.portal.system.IConstants;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.portal.system.init.IPhase;
import com.webmethods.portal.system.init.InitializationException;
import com.webmethods.rtl.event.BufferingQueue;
import com.webmethods.rtl.util.Debug;

import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This is the implementation of your {@link com.webmethods.portal.mech.IMechanics}
 *
 * @version $Revision: 1.0 $
 */
public class Related_pagesMechanics extends com.webmethods.portal.mech.portlet.impl.PortletMechanics {

    RelatedPagesCache m_cache = null;
    BufferingQueue m_delayedQueue = new BufferingQueue(100, true);

    Thread m_delayedProcess = new Thread() {
        public void run() {
            while (true) {
                try {
                    DelayedQueueEntry entry = (DelayedQueueEntry) m_delayedQueue.get();
                    IThingID currentResourceID = entry.currentResourceID;
                    String ignoredResources = entry.ignoredResources;
                    // we would like to skip over the ignored list and the current list in the my favorites
                    List ignorePages = ListTools.stringToList(ignoredResources);
                    for (int i = 0; i < ignorePages.size(); i++) {
                        String ignorePage = (String) ignorePages.get(i);
                        ignorePages.set(i, PortalSystem.getPortalSystem().acquireURI(ignorePage));
                    }

                    ignorePages.add(currentResourceID);

                    List result = new ArrayList();
                    ISqlWorker worker = null;
                    try {
                        worker = DataManager.createWorker();
                        Integer dbId = currentResourceID.getDbID();
                        ResultSet rs = worker.executeQueryProcResult("spLoadRelatedPages", new Object[]{dbId});
                        while (rs.next()) {
                            IThingID itemID = Atomic.createValidThingId(rs, 1, currentResourceID.getMetaContext());
                            if (!ignorePages.contains(itemID)) {
                                // result contains string uris
                                result.add(itemID.toString());
                            }
                        }
                    }
                    finally {
                        if (worker != null) {
                            worker.release();
                        }
                    }

                    Debug.info("Load related pages for: " + currentResourceID);
                    // populate cache
                    ICacheKey cacheKey = m_cache.createKey(currentResourceID.getDbID().intValue());
                    m_cache.put(cacheKey, 24 * 3600 * 1000, result);
                }
                catch (BufferingQueue.ShutdownException se) {
                    // ignore and exit the thread
                    break;
                }
                catch (Exception e) {
                    Debug.warn(e);
                }
            }
        }
    };

    static class DelayedQueueEntry {
        IThingID currentResourceID;
        String ignoredResources;

        public DelayedQueueEntry(IThingID currentResourceID, String ignoredList) {
            this.currentResourceID = currentResourceID;
            this.ignoredResources = ignoredList;
        }
    }

    public static class RelatedPagesCache extends DependentCache implements ICache {

        public RelatedPagesCache() {
        }
    }

    /**
     * Initialize the component with the appropriate componentData
     *
     * @param componentData which contains enough information for this component to initialize
     * @throws com.webmethods.portal.system.init.InitializationException
     *
     */
    public void init(IComponentData componentData, String phase) throws InitializationException {
        super.init(componentData, phase);

        try {
            if (m_cache == null) {
                ICacheManager cm = (ICacheManager) PortalSystem.getCacheProvider();
                Map props = new HashMap();
                props.put(ICache.ID, String.valueOf(ICacheManager.MAX_CACHE_ID - 1)); // make sure we don't interfere with internal caches
                props.put(IConstants.CLASS, RelatedPagesCache.class.getName());
                props.put(IConstants.NAME, "Related Pages");
                props.put(IConstants.ENABLED, "true");
                props.put(ICache.MAX_SIZE, "5000");
                props.put(ICache.ISCLUSTERED, "true");
                m_cache = (RelatedPagesCache) cm.createCache(RelatedPagesCache.class.getName(), "Related Pages", 5000,
                        true, IPhase.DATA_PHASE1, props);
            }
        }
        catch (PortalException e) {
            throw new InitializationException(e, InitializationException.RECOVERABLE_ERROR);
        }

        // start queue and thread
        if (!m_delayedProcess.isAlive()) {
            m_delayedProcess.start();
        }
    }

    /**
     * Allows your component to shutdown
     * This will clear the data structures, so do your work first
     */
    public void shutdown() {
        super.shutdown();
        m_delayedQueue.shutdown();
    }

    public List getRelatedPages(IThingID currentResourceID) {
        ICacheKey cacheKey = m_cache.createKey(currentResourceID.getDbID().intValue());
        ICacheEntry cacheEntry = m_cache.getEntry(cacheKey);
        return cacheEntry != null ? (List) cacheEntry.getValue() : null;
    }

    public void loadRelatedPages(IThingID currentResourceID, String ignoredResources) {
        m_delayedQueue.put(new DelayedQueueEntry(currentResourceID, ignoredResources));
    }
}