/**
 ** $Workfile: SpreadSheetDataSource.java $
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_spreadsheetdatasource;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.bizPolicy.IContext;
import com.webmethods.portal.framework.portlet.beans.JspPortletBean;
import com.webmethods.portal.mech.IMechanicsManager;
import com.webmethods.portal.mech.cache.IPresentationCacheMechanics;
import com.webmethods.portal.mech.storage.IContentMechanics;
import com.webmethods.portal.service.cache.ICacheKey;
import com.webmethods.portal.service.meta2.thing.IThingID;
import com.webmethods.portal.service.meta2.thing.impl.ThingIDList;
import com.webmethods.portal.system.IURI;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.rtl.util.ObjectUtil;
import com.webmethods.rtl.util.StringTools;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * SpreadSheetDataSource portlet implementation. When displaying
 * 
 * @version $Revision: 1.0 $
 */
public class SpreadSheetDataSource extends JspPortletBean {

	public final static String PROPERTY_DATASOURCE = "dataSource";

	public final static String PROPERTY_SHEETNAME = "sheetName";

	public final static String PROPERTY_DISPLAYCOLUMNTITLES = "displayColumnTitles";

	public final static String PROPERTY_COLUMNTITLESROW = "columnTitlesRow";

	public final static String PROPERTY_ROWSTODISPLAY = "rowsToDisplay";

	public final static String PROPERTY_COLUMNSTODISPLAY = "columnsToDisplay";

	public SpreadSheetData loadSpreadSheetData() throws Exception {
		String dataSource = getDataSource();
		if (StringTools.isEmpty(dataSource)) {
			return null;
		}

		IThingID dataSourceID = (IThingID) PortalSystem.getPortalSystem()
				.acquireURI(dataSource);
		if (dataSourceID == null) {
			return null;
		}

		SpreadSheetData data = getCacheSpreadsheetData();
		if (data != null) {
			return data;
		}

		IMechanicsManager mechManager = (IMechanicsManager) PortalSystem
				.getMechanicsProvider();
		IContentMechanics mech = (IContentMechanics) mechManager
				.getMechanics(dataSourceID);

		InputStream contentStream = mech.getContent(dataSourceID);
		HSSFWorkbook workbook = new HSSFWorkbook(contentStream);
		if (workbook == null) {
			return null;
		}

		String sheetName = getSheetName();
		HSSFSheet sheet = workbook.getSheet(sheetName);
		if (sheet == null) {
			return null;
		}

		boolean displayColumnTitles = getDisplayColumnTitlesAsBoolean();
		int columnTitlesRow = getColumnTitlesRowAsInt();
		String rowsToDisplayString = getRowsToDisplay();
		int[] rowsToDisplay = StringTools
				.getIntArrayFromCsv(rowsToDisplayString);
		String columnsToDisplayString = getColumnsToDisplay();
		int[] columnsToDisplay = StringTools
				.getIntArrayFromCsv(columnsToDisplayString);

		String[] titles = null;
		List rowsOfData = new ArrayList();
		int index = 1;
		for (Iterator it = sheet.rowIterator(); it.hasNext(); index++) {
			HSSFRow row = (HSSFRow) it.next();
			if (index == columnTitlesRow && displayColumnTitles) {
				titles = getRowContent(columnsToDisplay, row);
			} else {
				for (int i = 0; i < rowsToDisplay.length; i++) {
					int i1 = rowsToDisplay[i];
					if (index == i1) {
						String[] content = getRowContent(columnsToDisplay, row);
						rowsOfData.add(content);
					}
				}
			}
		}
		data = new SpreadSheetData(titles, rowsOfData);
		putCacheSpreadsheetData(dataSourceID, data);
		return data;
	}

	public static class SpreadSheetData implements Serializable {
		private String[] titles;

		private List rowsOfData;

		public SpreadSheetData(String[] titles, List rowsOfData) {
			this.titles = titles;
			this.rowsOfData = rowsOfData;
		}

		public String[] getTitles() {
			return titles;
		}

		public List getRowsOfData() {
			return rowsOfData;
		}
	}

	private String[] getRowContent(int[] columnsToDisplay, HSSFRow row) {
		String[] content = new String[columnsToDisplay.length];
		for (int index = 0; index < columnsToDisplay.length; index++) {
			int column = columnsToDisplay[index] - 1; // user input is 1 based
			HSSFCell cell = row.getCell((short) column);
			String value = "";
			if (cell != null) {
				if (cell.getCellType() == HSSFCell.CELL_TYPE_NUMERIC) {
					value = String.valueOf(cell.getNumericCellValue());
				} else {
					value = cell.getStringCellValue();
				}
			}
			content[index] = value;
		}
		return content;
	}

	/**
	 * Release any local variables so the object can be added back into the
	 * object pool.
	 */
	public void release() {
		super.release();
	}

	/**
	 * Sets the value of the 'dataSource' property.
	 * 
	 * @param value
	 *            the value to use for the 'dataSource' property
	 */
	public void setDataSource(java.lang.String value) {
		setProperty(PROPERTY_DATASOURCE, value);
	}

	/**
	 * Gets the value of the 'dataSource' property.
	 * 
	 * @return the value of the 'dataSource' property
	 */
	public java.lang.String getDataSource() throws PortalException {
		String strValue = getPropertyAsString(PROPERTY_DATASOURCE, null);
		if (StringTools.isEmpty(strValue)) { // null check
			return null;
		}
		return strValue;
	};

	/**
	 * Sets the value of the 'sheetName' property.
	 * 
	 * @param value
	 *            the value to use for the 'sheetName' property
	 */
	public void setSheetName(java.lang.String value) {
		setProperty(PROPERTY_SHEETNAME, value);
	}

	/**
	 * Gets the value of the 'sheetName' property.
	 * 
	 * @return the value of the 'sheetName' property
	 */
	public java.lang.String getSheetName() throws PortalException {
		String strValue = getPropertyAsString(PROPERTY_SHEETNAME, null);
		if (StringTools.isEmpty(strValue)) { // null check
			return null;
		}
		return strValue;
	};

	/**
	 * Sets the value of the 'displayColumnTitles' property.
	 * 
	 * @param value
	 *            the value to use for the 'displayColumnTitles' property
	 */
	public void setDisplayColumnTitles(java.lang.String value) {
		setProperty(PROPERTY_DISPLAYCOLUMNTITLES, value);
	}

	/**
	 * Gets the value of the 'displayColumnTitles' property.
	 * 
	 * @return the value of the 'displayColumnTitles' property
	 */
	public java.lang.String getDisplayColumnTitles() throws PortalException {
		String strValue = getPropertyAsString(PROPERTY_DISPLAYCOLUMNTITLES,
				null);
		if (StringTools.isEmpty(strValue)) { // null check
			return null;
		}
		return strValue;
	};

	public boolean getDisplayColumnTitlesAsBoolean() throws PortalException {
		return getPropertyAsBoolean(PROPERTY_DISPLAYCOLUMNTITLES);
	}

	/**
	 * Sets the value of the 'columnTitlesRow' property.
	 * 
	 * @param value
	 *            the value to use for the 'columnTitlesRow' property
	 */
	public void setColumnTitlesRow(java.lang.String value) {
		setProperty(PROPERTY_COLUMNTITLESROW, value);
	}

	/**
	 * Gets the value of the 'columnTitlesRow' property.
	 * 
	 * @return the value of the 'columnTitlesRow' property
	 */
	public java.lang.String getColumnTitlesRow() throws PortalException {
		String strValue = getPropertyAsString(PROPERTY_COLUMNTITLESROW, null);
		if (StringTools.isEmpty(strValue)) { // null check
			return null;
		}
		return strValue;
	};

	public int getColumnTitlesRowAsInt() throws PortalException {
		return getPropertyAsInt(PROPERTY_COLUMNTITLESROW);
	}

	/**
	 * Sets the value of the 'rowsToDisplay' property.
	 * 
	 * @param value
	 *            the value to use for the 'rowsToDisplay' property
	 */
	public void setRowsToDisplay(java.lang.String value) {
		setProperty(PROPERTY_ROWSTODISPLAY, value);
	}

	/**
	 * Gets the value of the 'rowsToDisplay' property.
	 * 
	 * @return the value of the 'rowsToDisplay' property
	 */
	public java.lang.String getRowsToDisplay() throws PortalException {
		String strValue = getPropertyAsString(PROPERTY_ROWSTODISPLAY, null);
		if (StringTools.isEmpty(strValue)) { // null check
			return null;
		}
		return strValue;
	};

	/**
	 * Sets the value of the 'columnsToDisplay' property.
	 * 
	 * @param value
	 *            the value to use for the 'columnsToDisplay' property
	 */
	public void setColumnsToDisplay(java.lang.String value) {
		setProperty(PROPERTY_COLUMNSTODISPLAY, value);
	}

	/**
	 * Gets the value of the 'columnsToDisplay' property.
	 * 
	 * @return the value of the 'columnsToDisplay' property
	 */
	public java.lang.String getColumnsToDisplay() throws PortalException {
		String strValue = getPropertyAsString(PROPERTY_COLUMNSTODISPLAY, null);
		if (StringTools.isEmpty(strValue)) { // null check
			return null;
		}
		return strValue;
	};

	private SpreadSheetData getCacheSpreadsheetData() throws PortalException {
		IContext context = getPortletContext();
		ICacheKey key = getCacheKey(context, getPortletThingID());
		IPresentationCacheMechanics mech = getCacheMechanics();
		byte[] bytes = mech.getContent(key);
		if (bytes == null) {
			return null;
		}

		SpreadSheetData spreadSheetData = (SpreadSheetData) ObjectUtil
				.load(bytes);
		return spreadSheetData;
	}

	private void putCacheSpreadsheetData(IThingID dataSourceID,
			SpreadSheetData spreadSheetData) throws PortalException {
		IContext context = getPortletContext();
		IThingID portletThingID = getPortletThingID();
		ICacheKey key = getCacheKey(context, portletThingID);
		byte[] bytes = ObjectUtil.save(spreadSheetData);
		ThingIDList thingIDList = new ThingIDList();
		thingIDList.add((IThingID) portletThingID);
		thingIDList.add(dataSourceID);
		IPresentationCacheMechanics mech = getCacheMechanics();
		mech.putContent(key, bytes, thingIDList);
	}

	/**
	 * Share key among all users
	 */
	private ICacheKey getCacheKey(IContext context, IURI spreadsheetURI)
			throws PortalException {
		IPresentationCacheMechanics mech = getCacheMechanics();
		ICacheKey key = mech.getKey(context.getUserID().getMetaContext()
				.getUserService().getAdminUserID(), (IThingID) spreadsheetURI,
				"");
		return key;
	}
}
