/**
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 **/
package com.webmethods.portal.portlet.wm_auditcommands.mechanics;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.bizPolicy.IContext;
import com.webmethods.portal.bizPolicy.impl.ContextFactory;
import com.webmethods.portal.event.ISystemPropertyEvent;
import com.webmethods.portal.event.alias.IAliasChangeEvent;
import com.webmethods.portal.event.dir.IDirServiceEvent;
import com.webmethods.portal.event.modify.permissions.IPermissionsEvent;
import com.webmethods.portal.event.remove.IDeleteEvent;
import com.webmethods.portal.event.system.ILoginEvent;
import com.webmethods.portal.event.system.ILoginFailedEvent;
import com.webmethods.portal.mech.portlet.impl.PortletMechanics;
import com.webmethods.portal.system.IComponentData;
import com.webmethods.portal.system.init.InitializationException;
import com.webmethods.rtl.event.IEvent;
import com.webmethods.rtl.util.Debug;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * This class (like all mechanics) are singletons.  At init, it registers to listen for different types
 * of events (synchronously) and logs all local events.  In other words, it doesn't log events that
 * were triggered remotely
 */
public class AuditCommandsMechanics extends PortletMechanics implements ISystemPropertyEvent.ISynchronousListener,
        IAliasChangeEvent.ISynchronousListener, IDirServiceEvent.ISynchronousListener, IPermissionsEvent.ISynchronousListener,
        IDeleteEvent.ISynchronousListener, ILoginFailedEvent.ISynchronousListener {

    private static Logger s_logger = null;

    public void init(IComponentData componentData, String phase) throws InitializationException {
        super.init(componentData, phase);
        PropertyConfigurator.configure(AuditCommandsMechanics.class.getResource("log4j.properties"));
		s_logger = Logger.getLogger("auditCommands");
        ISystemPropertyEvent.SynchronousListeners.add(this);
        IAliasChangeEvent.SynchronousListeners.add(this);
        IDirServiceEvent.SynchronousListeners.add(this);
        IPermissionsEvent.SynchronousListeners.add(this);
        IDeleteEvent.SynchronousListeners.add(this);
        ILoginFailedEvent.SynchronousListeners.add(this);
    }

    public void shutdown() {
        ISystemPropertyEvent.SynchronousListeners.remove(this);
        IAliasChangeEvent.SynchronousListeners.remove(this);
        IDirServiceEvent.SynchronousListeners.remove(this);
        IPermissionsEvent.SynchronousListeners.remove(this);
        IDeleteEvent.SynchronousListeners.remove(this);
        ILoginFailedEvent.SynchronousListeners.remove(this);
        super.shutdown();
    }

    public void onSynchronousEvent(ISystemPropertyEvent ev) {
        if (!ev.isLocalEvent()) {
            return;
        }

        auditEvent(ev, "system property changed: " + ev.getDifferenceEntry().toString());
    }

    public void onSynchronousEvent(IAliasChangeEvent ev) {
        if (!ev.isLocalEvent()) {
            return;
        }

        String action = ev.getAction() == IAliasChangeEvent.ACTION_ADD ? "added" : "removed";
        auditEvent(ev, "alias was " + action + ": " + ev.getKey() + "->" + ev.getValue());
    }

    public void onSynchronousEvent(IDirServiceEvent ev) {
        if (!ev.isLocalEvent()) {
            return;
        }

        String action = ev.getDirServiceAction() == IDirServiceEvent.ADDED ? "action" :
                ev.getDirServiceAction() == IDirServiceEvent.REMOVED ? "removed" : "modified";

        String name = null;
        try {
            name = ev.getDirServiceID().getThing().getName();
        } catch (Exception e) {
            name = "unknown directory service";
        }
        auditEvent(ev, name + " was " + action);
    }

    public void onSynchronousEvent(IPermissionsEvent ev) {
        if (!ev.isLocalEvent()) {
            return;
        }

        auditEvent(ev, "Permissions were modified for: " + ev.getThingId());
    }

    public void onSynchronousEvent(IDeleteEvent ev) {
        if (!ev.isLocalEvent()) {
            return;
        }

        String name = null;
        try {
            name = ev.getDeletedID().getThing().getName();
        } catch (Exception e) {
            name = "unknown";
        }

        auditEvent(ev, name + " was deleted");
    }

    public void onSynchronousEvent(ILoginEvent ev) {
        if (!ev.isLocalEvent()) {
            return;
        }
        if (!(ev instanceof ILoginFailedEvent)) {
            return;
        }
        ILoginFailedEvent loginFailedEvent = (ILoginFailedEvent) ev;

        if (ev.getAction() == ILoginEvent.ACTION_FAILED_LOGIN) {
            auditEvent(ev, loginFailedEvent.getLoginName() + " failed to login");
        }
    }

    private void auditEvent(IEvent event, String message) {
        String userName = null;
        try {
            IContext context = ContextFactory.acquireContext(true);
            userName = context.getUserName();
        } catch (PortalException e) {
            userName = "unknown";
        }

        Debug.info("Event: " + event.getClass() + " " + message + " by user: " + userName, s_logger);
    }
}