/**
 ** $Workfile: SecurityRedrect.java $
 ** Copyright (c) 2001-2005 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_securityredirect;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.text.MessageFormat;

import com.webmethods.portal.PortalException;
import com.webmethods.portal.bizPolicy.IContext;
import com.webmethods.portal.bizPolicy.biz.IBizPolicyManager;
import com.webmethods.portal.bizPolicy.biz.IBizPolicyNames;
import com.webmethods.portal.bizPolicy.biz.dir.IPrincipalBizPolicy;
import com.webmethods.portal.bizPolicy.impl.ContextFactory;
import com.webmethods.portal.framework.portlet.beans.JspPortletBean;
import com.webmethods.portal.framework.presentation.PresentationData;
import com.webmethods.portal.service.dir.IDirConstants;
import com.webmethods.portal.service.dir.IDirPrincipal;
import com.webmethods.portal.service.dir.IDirPrincipalList;
import com.webmethods.portal.service.dir.IDirSystem;
import com.webmethods.portal.service.dir.IDirUser;
import com.webmethods.portal.system.IURI;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.rtl.util.PathUtil;
import com.webmethods.rtl.util.StringTools;

public class SecurityRedirect extends JspPortletBean {

    public final static String PROPERTY_PRIVILEGEDPRINCIPALS = "privilegedPrincipals";
	public final static String PROPERTY_REDIRECTRESOURCE = "redirectResource";
	
	public final static String REDIRECT_TEMPLATE = "<script>window.location=\"{0}\";</script>";
	public final static MessageFormat REDIRECT_FORMAT = new MessageFormat(REDIRECT_TEMPLATE);

	public boolean shouldRedirect() throws PortalException {
		//admin check
		IContext context = ContextFactory.acquireContext(getRequest());
		if (context.isAdminSession()) {
			return false;
		}
		
		String principalString = getPrivilegedPrincipals();
		if (StringTools.isEmpty(principalString)) {
			return true;
		}
		
		IDirSystem dirSystem = (IDirSystem)PortalSystem.getDirectoryProvider();
		IBizPolicyManager bizPolicyManager = (IBizPolicyManager)PortalSystem.getBizPolicyProvider();
		IPrincipalBizPolicy princBizPolicy = (IPrincipalBizPolicy) bizPolicyManager.getBizPolicy(IBizPolicyNames.PRINCIPAL);
		
		IDirUser currentUser = (IDirUser)dirSystem.lookup(context.getUserID()); 

		String [] principalStringArray = StringTools.getArrayFromCsvString(principalString);
		for (String principalStr : principalStringArray) {
			IURI dirUri = PortalSystem.getPortalSystem().acquireURI(principalStr);
			IDirPrincipal dirPrincipal = (IDirPrincipal)dirSystem.lookup(dirUri);
			
			if (dirPrincipal != null) {
				switch (dirPrincipal.getType()) {
				case IDirConstants.TYPE_USER:
					if (currentUser.equals(dirPrincipal)) {
						return false;
					}
					break;
				case IDirConstants.TYPE_GROUP:
				case IDirConstants.TYPE_ROLE:
					IDirPrincipalList list = princBizPolicy.getMembers(context, dirPrincipal.getDirectoryURI());
					if (list != null && list.contains(currentUser)) {
						return false;
					}
					break;
				}
			}
		}
		
		return true;
	}
	
	protected Reader getContentInternal() throws IOException, PortalException {
		String content = null;
		if (shouldRedirect()) {
        	String url = getRedirectURL();
			content = REDIRECT_FORMAT.format(new String[]{url});
        } else {
        	content = new String();
        }
        return new StringReader(content);
    }
	
	private String getRedirectURL() throws PortalException {
		PresentationData data = PresentationData.getPresentationData(getRequest());
		String requestURL = data.getRequestedServerURL();
		String redirectResource = getRedirectResource();
		return PathUtil.appendPath(requestURL, redirectResource);
	}
	
	/**
	 * Sets the value of the 'privilegedPrincipals' property.
	 * 
	 * @param value
	 *            the value to use for the 'privilegedPrincipals' property
	 */
    public void setPrivilegedPrincipals(java.lang.String value) {
        setProperty(PROPERTY_PRIVILEGEDPRINCIPALS, value);
    }

	/**
	 * Gets the value of the 'privilegedPrincipals' property.
	 * 
	 * @return the value of the 'privilegedPrincipals' property
	 */
    public java.lang.String getPrivilegedPrincipals() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_PRIVILEGEDPRINCIPALS, null);
        if (StringTools.isEmpty(strValue)) { // null check
            return null;    
        }
        return strValue;
    }

	/**
	 * Sets the value of the 'redirectResource' property.
	 * 
	 * @param value
	 *            the value to use for the 'redirectResource' property
	 */
    public void setRedirectResource(java.lang.String value) {
        setProperty(PROPERTY_REDIRECTRESOURCE, value);
    }

	/**
	 * Gets the value of the 'redirectResource' property.
	 * 
	 * @return the value of the 'redirectResource' property
	 */
    public java.lang.String getRedirectResource() throws PortalException {
        String strValue = getPropertyAsString(PROPERTY_REDIRECTRESOURCE, null);
        if (StringTools.isEmpty(strValue)) { // null check
            return null;    
        }
        return strValue;
    };

}
