/**
 ** $Workfile: Dbdir_auth_template.java $
 ** Copyright (c) 2001-2004 webMethods, Inc. All Rights Reserved.
 */
package com.webmethods.portal.portlet.wm_dbdir_auth_template;

import com.webmethods.portal.framework.portlet.beans.JspPortletBean;
import com.webmethods.portal.service.dir.*;
import com.webmethods.portal.service.meta2.thing.IThing;
import com.webmethods.portal.service.sql.ISqlWorker;
import com.webmethods.portal.service.sql.DataManager;
import com.webmethods.portal.PortalException;
import com.webmethods.portal.system.PortalSystem;
import com.webmethods.portal.mech.IMechanicsManager;
import com.webmethods.rtl.util.ObjectUtil;
import com.webmethods.rtl.util.Debug;

import java.sql.ResultSet;
import java.text.CharacterIterator;
import java.text.StringCharacterIterator;

/**
 * Dbdir_auth_template portlet implementation.
 *
 * @version    $Revision: 1.0 $
 */
public class Dbdir_auth_template extends JspPortletBean implements IDirAuthenticator {
    static final String ATTRIB_UID = "\\{uid\\}";
    static final String ATTRIB_DN = "\\{dn\\}";
    static final String ESCAPERS = "\\$"; 
	static final char BACKSLASH = '\\';

    /**
     * Attempts to authenticate the IDirectoryUser with the supplied credentials.
     * If this succeeds, then the function will return, otherwise an Authentication Exception
     * will be thrown.
     *
     * @param user
     * @param credentials
     * @throws com.webmethods.portal.PortalException
     *
     * @throws com.webmethods.portal.service.dir.AuthenticationException
     *
     */
    public void authenticateUser(IDirUser user, Object credentials) throws PortalException, AuthenticationException {
        // get database directory service
        IDirSystem ds = (IDirSystem) PortalSystem.getDirectoryProvider();
        IDirService dirService = ds.getDirectoryServiceFromThingID(user.getDirectoryServiceThingID());
        IDirConfig dirConfig = dirService.getDirConfig();

        // get query to execute
        String datasource = (String) dirConfig.getProperty("datasource");
        String authenticateQuery = (String) dirConfig.getProperty("authenticate");
        // replace parameters UserID and DN
        authenticateQuery = authenticateQuery.replaceAll(ATTRIB_DN, 
        		escapeSpecialChars(sqlEscape(user.getDN()),ESCAPERS));
        authenticateQuery = authenticateQuery.replaceAll(ATTRIB_UID, 
        		escapeSpecialChars(sqlEscape(user.getName()),ESCAPERS));

        ISqlWorker worker = null;
        boolean authenticated = false;
        try {
            // fetch clear text password and compare them
            worker = DataManager.createWorker(datasource);
            ResultSet rs = worker.executeQuery(authenticateQuery);
            if (rs.next()) {
                String password = rs.getString(1);
                authenticated = password.equals(credentials);
            }
        }
        catch (Exception e) {
            Debug.fatal(e);
        }
        finally {
            if (worker != null) {
                worker.release();
            }
        }

        if (!authenticated) {
            throw new AuthenticationException(user.getName());
        }
    }

    protected String sqlEscape(String value) {
        if (value.indexOf("'") >= 0) {
            StringBuffer b = new StringBuffer(value.length());
            for (int i = 0; i < value.length(); i++) {
                char ch = value.charAt(i);
                if (ch != '\'') {
                    b.append(ch);
                }
                else {
                    b.append("''");
                }
            }

            return b.toString();
        }
        else return value;
    }
    
    protected String escapeSpecialChars(String value, String specialChars){
		String result = value;
		if(value == null || specialChars == null)
			return result;
		StringCharacterIterator sci = new StringCharacterIterator(specialChars);
		char specialChar = sci.first();
        boolean backslashEscaped = false;
        while (specialChar != CharacterIterator.DONE) {
            if (specialChar == BACKSLASH && !backslashEscaped) { //escape backslash only once
                result = escape(result, specialChar, BACKSLASH); //escape with leading backslash
                backslashEscaped = true;
            } else {
                result = escape(result, specialChar, BACKSLASH); //escape with leading backslash
            }
            specialChar = sci.next();
        }//next character
        return result;
	}//escapeSpecialChars()
    
    protected String escape(String string, char character, char escape) {
        String result = string; //default;
        if (string != null) {
            StringBuffer sb = new StringBuffer();
            StringCharacterIterator sci = new StringCharacterIterator(string);
            char c = sci.first();
            while (c != CharacterIterator.DONE) {
                if (c == character) {
                    sb.append(escape);
                }
                sb.append(c);
                c = sci.next();
            }//next character
            result = sb.toString();
        }
        return result;
    }//escape()

}
